/* Testiranje AVL rutin. LK 1994 */

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include "AVLtree.h"

struct ELEMENT
{
  int vsebina;
  char *tekst;
};

/*FCN Funkcija potrebna za primerjanje elementov */
int
primerjaj (struct ELEMENT * prvi, struct ELEMENT * drugi)
{
  if (prvi->vsebina > drugi->vsebina)
    return (1);
  if (prvi->vsebina < drugi->vsebina)
    return (-1);
  return (0);
}


/*FCN funkcija potrebna za brisanje elementa iz drevesa */
void
brisi (struct ELEMENT * element)
{
  if (element->tekst)
    free (element->tekst);
  free (element);
}

/*FCN Izpise element na zaslon*/
void
izpis (struct ELEMENT * element)
{
  printf ("%s ", element->tekst);
}


int
main (void)
{
#define TOTAL 10

  static int data[TOTAL] =
  {1, 4, 3, 5, 9, 2, 3, 5, 7, 0};
  static char *tekst[TOTAL] =
  {"ena", "stiri", "tri", "pet",
   "devet", "dve", "tri", "pet", "sedem", "nic"};

  AVLtree *root = NULL;
  struct ELEMENT *element, iskani =
  {3, NULL};
  int i, status;

  for (i = 0; i < TOTAL; i++)	/* zgradimo testno drevo */
    {
      element = (struct ELEMENT *) malloc (sizeof (struct ELEMENT));
      element->vsebina = data[i];
      element->tekst = strdup (tekst[i]);
      status = AVLinsert (element, &root, primerjaj);
      printf ("Vstavil sem element %d %s, \tstatus = %d\n",
	      data[i], tekst[i], status);
    }

  printf ("Brisem element %d, status = %d\n", iskani.vsebina,
	  AVLdelete (&iskani, &root, primerjaj, brisi));

  element = AVLfind (&iskani, root, primerjaj);
  if (element == NULL)
    printf ("Nisem nasel iskanega elementa!\n");
  else
    printf ("Nasel sem iskani element %d, %s\n",
	    element->vsebina, element->tekst);

  printf ("Preorder: ");
  AVLpreorder (root, izpis);
  printf ("\nInorder: ");
  AVLinorder (root, izpis);
  printf ("\nPostorder: ");
  AVLpostorder (root, izpis);
  printf ("\nBrisanje drevesa:\n");
  AVLrelease (&root, brisi);

  return (0);
}
