/***************************************************************************
   Module Name:  btree      sdk_src_btree_btree.c

   Description:  Binary Tree 

   Author:  Tools and Technologies Group
            Autodesk, Inc.
            Software Developers' Kit


     (C) Copyright 1990, 1991 by Autodesk, Inc.
***************************************************************************
*  Permission to use, copy, modify, and distribute this software and its  *
*  documentation for the purpose of creating applications for AutoCAD, is *
*  hereby granted in accordance with the terms of the License Agreement   *
*  accompanying this product.                                             *
*                                                                         *
***************************************************************************
*                                                                         *
*  AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.  AUTODESK  *
*  SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY OR      *
*  FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. DOES NOT WARRANT THAT    *
*  THE OPERATION OF THE PROGRAM WILL BE UNINTERRUPTED OR ERROR FREE.      *
*                                                                         *
***************************************************************************
*                                                                         *
*                       RESTRICTED RIGHTS LEGEND                          *
*                                                                         *
*  Use, duplication, or disclosure by the U.S. Government is subject      *
*  to restrictions set forth in FAR 52.227-19 (Commercial Computer        *
*  Software - Restricted Rights) and DFAR 252.227-7013 (c) (1) (ii)       *
*  (Rights in Technical Data and Computer Software), as applicable.       *
***************************************************************************
*                                                                         *
*   Last Modification:                                                    *
*                                                                         *
*   Notes and restrictions on use:                                        *
***************************************************************************/  
/**************************************************************************/
/* STANDARD C INCLUDE FILES */
/**************************************************************************/
#include  <stdio.h>
#include  "sdk.h"
#include  "btree.h"


/**************************************************************************/
/*  GLOBAL VARIABLES  */
/**************************************************************************/
nodetype               *BT_HEAD     = NULL; /*  head of btree  */
static nodetype        *bt_cnode    = NULL; /*  current node pointer  */
static char            *bt_fkey     = NULL; /*  global find key  */
static int             (*bt_func)();        /*  global compare function  */
static listtype        *BT_LIST     = NULL; /*  btree set list  */
static short           bt_test      = 0;    /*  compare test flag  */
static listtype        *bt_tlist    = NULL; /*  temporary set list ptr  */
static nodetype        *bt_tnode    = NULL; /*  temporary node pointer  */


/***************************************************************************/
/*  GLOBAL FUNCTIONS  */
/**************************************************************************/
nodetype   *bt_allocnode();
nodetype   *bt_findnode();
int        bt_findtest();
void       bt_lstbld();
nodetype   *bt_lstget();
void       bt_node();
int        bt_true();
char       *malloc();
void       free();


/***************************************************************************/
/*.doc bt_allocnode(user)*/
/*+
   Allocates a node for the btree and stores the specified key in the node.
   The node is initialised as a NULL node and is not inserted in the btree.
   If the function fails to allocate a node, then it returns NULL.
-*/
/***************************************************************************/
nodetype 
/* FCN */*bt_allocnode(key)
  char *key;                        /* (I) key to store in node */
{

    /*  ALLOCATE A NODE  */
    if ((bt_tnode = (nodetype *)malloc(sizeof(nodetype))) == NULL) {
        return((nodetype *)NULL);
    }

    /*  INITIALIZE THE NODE AS A NULL NODE  */
    bt_tnode->l = bt_tnode->r = bt_tnode->p = (nodetype *)NULL;
    bt_tnode->flags.ltag = bt_tnode->flags.rtag = bt_tnode->flags.delete = 0;
    bt_tnode->key = key;
   
    return bt_tnode;                /*  RETURN NODE POINTER TO CALLER  */
}                                   /*  END BT_ALLOCNODE FUNCTION*/


/***************************************************************************/
/*.doc bt_erasenode(user)*/
/*+
   Erases a node from the btree.
-*/
/***************************************************************************/
/* FCN */ bt_erasenode(node)
  nodetype   *node;                 /*  (I) pointer to node to erase  */
{
   if (node == NULL)
    return BAD;
    
   node->flags.delete = 1;

   return GOOD;
}

          
/***************************************************************************/
/*.doc bt_findnode(user)*/
/*+
   This function returns a pointer to a node based on the search parameters
   specified in the arguments.  The op argument maybe one of the following:
   '<', '>', or '='.  The first parameter should initially be called with a
   value of TRUE.  This identifies the search as the search for the first
   item in the tree matching the specified key.  Subsequent calls to the
   function should be called with first = FALSE, and this should be called
   until the function returns NULL.  This will successfully free internal
   memory which is allocated.
-*/
/***************************************************************************/
nodetype 
/* FCN */*bt_findnode(op, key, f, first)
  char       op;                    /* (I) operator */
  char       *key;                  /* (I) key to match on */
  int        (*f)();                /* (I) comparison function */
  int        first;                 /* (I) first (TRUE) or next */
{
    register int   i;               /* compare flag */


    /*  CHECK THAT WE"VE GOT A NON-EMPTY BTREE */
    if (BT_HEAD == NULL) 
        return(BT_HEAD);


    /* SET UP IF FIRST, ELSE RETURN LIST ENTRY */
    if (first) {
        while (BT_LIST) {
            bt_tlist = BT_LIST->next;
            free((char *)BT_LIST);
            BT_LIST = bt_tlist;
        }
        bt_cnode    = BT_HEAD;
        bt_func     = f;
        bt_fkey     = key;
    } else 
        return (bt_lstget());


    /* DETERMINE SEARCH PATH */
    switch (op) {
        case '>':   /*  FIND ALL NODES > key  */
            if ((*bt_func)(BT_HEAD->key,bt_fkey) == (bt_test = 1)) {
                while ((*bt_func)(bt_cnode->key,bt_fkey) == bt_test) {
                    if (bt_cnode->flags.rtag == 0) 
                        bt_cnode = bt_cnode->r;
                    else 
                        break;
                }
                while (bt_cnode != BT_HEAD) {
                    if ((*bt_func)(bt_cnode->key,bt_fkey) == bt_test)
                       bt_lstbld(bt_cnode);
                    bt_cnode = bt_cnode->l;
                }
                bt_lstbld(BT_HEAD);
                if (BT_HEAD->flags.ltag == 0) 
                   bt_node(BT_HEAD->l,'A',bt_true,bt_lstbld);
            } else {
                while ((*bt_func)(bt_cnode->key,bt_fkey) != bt_test) {
                    if (bt_cnode->flags.ltag == 0) 
                        bt_cnode = bt_cnode->l;
                    else 
                        break;
                }
                bt_node(bt_cnode,'A',bt_findtest,bt_lstbld);
            }
            bt_cnode = bt_lstget();
            break;

        case '<':
            if ((*bt_func)(BT_HEAD->key,bt_fkey) == (bt_test = -1)) {
                while ((*bt_func)(bt_cnode->key,bt_fkey) == bt_test) {
                    if (bt_cnode->flags.ltag == 0) 
                        bt_cnode = bt_cnode->l;
                    else 
                        break;
                }
                while (bt_cnode != BT_HEAD) {
                    if ((*bt_func)(bt_cnode->key,bt_fkey) == bt_test)
                        bt_lstbld(bt_cnode);
                    bt_cnode = bt_cnode->r;
                }
                bt_lstbld(BT_HEAD);
                if (BT_HEAD->flags.rtag == 0) 
                    bt_node(BT_HEAD->r,'D',bt_true,bt_lstbld);
            } else {
                while ((*bt_func)(bt_cnode->key,bt_fkey) != bt_test) {
                    if (bt_cnode->flags.rtag == 0) 
                        bt_cnode = bt_cnode->r;
                    else 
                        break;
                }
                bt_node(bt_cnode,'D',bt_findtest,bt_lstbld);
            }
            bt_cnode = bt_lstget();
            break;

       default:
       case '=':
           for (; bt_cnode!=NULL; ) {
               if ((i = (*f)(bt_cnode->key,key)) == 0) bt_lstbld(bt_cnode);
               else if (i == 1) 
                   bt_cnode = (bt_cnode->flags.rtag == 1) ? NULL : bt_cnode->r;
               else             
                   bt_cnode = (bt_cnode->flags.ltag == 1) ? NULL : bt_cnode->l;
           }
           bt_cnode = bt_lstget();
           break;
    }
 
    return(bt_cnode);               /*  RETURN CNODE NODE  */
}


/***************************************************************************/
/*.doc bt_findtest(internal)*/
/*+
-*/
/***************************************************************************/
/* FCN */ bt_findtest(node)
  nodetype *node;
{
    if ((*bt_func)(node->key,bt_fkey) == bt_test) 
        return TRUE;
    else 
        return FALSE;
}




/***************************************************************************/
/*.doc bt_lstbld(internal)*/
/*+
   This function is used to build a list of nodes.  The list header can
   be found in BT_LIST.  This list is designed to allow the creation of
   a candidate set of nodes.  It is used in the find function to build the
   list of all nodes matching the request.
-*/
/***************************************************************************/
void 
/* FCN */ bt_lstbld(node)

nodetype *node;
{


/*  ALLOCATE MEMORY FOR A LIST CELL  */

   bt_tlist = (listtype *)malloc(sizeof(listtype));


/*  INITIALIZE THE CELL TO POINT TO THE NODE  */

   bt_tlist->node = node;


/*  INSERT CELL IN LIST  */

   bt_tlist->next = (BT_LIST != NULL) ? BT_LIST : NULL;
   BT_LIST        = bt_tlist;


/*  RETURN TO CALLER  */

   return;


/*  END BT_LSTBLD FUNCTION  */
}




/***************************************************************************/
/*.doc bt_lstget(internal)*/
/*+
   This function is used to return the first node from the list created by
   bt_lstbld.  The cell pointing to the node is 'popped' from the list.
-*/
/***************************************************************************/
nodetype 
/* FCN */ *bt_lstget()

{


/*  IF THERE IS SOMETHING IN THE LIST, RETURN THE FIRST NODE  */

   while (BT_LIST) {
       if (BT_LIST->node->flags.delete == 0) {
           bt_tnode = BT_LIST->node;
           bt_tlist = BT_LIST->next;
           free((char *)BT_LIST);
           BT_LIST  = bt_tlist;
           return bt_tnode;
       }
       bt_tlist = BT_LIST->next;
       free((char *)BT_LIST);
       BT_LIST  = bt_tlist;
   }


/*  ELSE RETURN NULL POINTER  */

   return(NULL);


/*  END BT_LSTGET FUNCTION  */
}




/***************************************************************************/
/*.doc bt_freenode(user)*/
/*+
   Frees up the memory allocated for a node.  The node must have been
   erased from the btree using the function bt_erasenode prior to any call 
   to this function.  If the node has not been erased, the integrity of 
   the btree will be lost.  The funciton returns BAD if the node has not
   been deleted.
-*/
/***************************************************************************/
/* FCN */ bt_freenode(node)
  nodetype *node;                   /*  (I) node to be deallocated  */
{
    /*  CHECK THAT THE NODE HAS BEEN DELETED BEFORE PROCEEDING  */

    if (node->flags.delete != 1) return BAD;


    /*  DEALLOCATED THE NODE IF IT IS AT THE ENDOF THE THREAD */

    if (node->flags.rtag == 1 && node->flags.ltag == 1) {
        if (node->p == NULL) {
            BT_HEAD = bt_cnode = (nodetype *)NULL;
        }
        else if (node->p->r == node) {
            node->p->r = node->r;
            node->p->flags.rtag = 1;
        }
        else if (node->p->l == node) {
            node->p->l = node->l;
            node->p->flags.ltag = 1;
        }

        free((char *)node);
    }

    return GOOD;
}                                   /*  END BT_FREENODE FUNCTION  */




/***************************************************************************/
/*.doc bt_node(user)*/
/*+
   This routine traverses the btree in either ascending or descending
   order depending on the setting of the 'd' argument.  'n' is a pointer
   to a node upon which the function 'f' is to act after the appropriate
   branch of the tree has been acted upon.  If 'd' is 'A', then
   ascending order is performed.  If 'd' is 'D', then descending
   order is performed.  Otherwise, the operation is undefined.  'f' is
   called with a pointer to a node as it's only argument.
-*/
/***************************************************************************/
void
/* FCN */ bt_node(n, d, t, f)
  nodetype   *n;                    /*  (I) pointer to node to act on  */
  char       d;                     /*  (I) direction  */
  int        (*t)();                /*  (I) test function  */
  int        (*f)();                /*  (I) pointer to function  */
{
    /* PROCESS THE TREE IN ORDER BASED ON THE REQUESTED ORDER */
    switch (d) {
        case 'A':                   /*  ASCENDING  */
            if (n->flags.rtag == 0 && (*t)(n)) bt_node(n->r,d,t,f);
            if ((*t)(n)) (*f)(n);
            if (n->flags.ltag == 0 && (*t)(n)) bt_node(n->l,d,t,f);
            break;

        case 'D':                   /*  DESCENDING  */
            if (n->flags.ltag == 0 && (*t)(n)) bt_node(n->l,d,t,f);
            if ((*t)(n)) (*f)(n);
            if (n->flags.rtag == 0 && (*t)(n)) bt_node(n->r,d,t,f);
            break;

        default:                    /*  UNDEFINED  */
           break;
   }
}                                   /*  END BT_NODE FUNCTION  */


/***************************************************************************/
/*.doc bt_storenode(user)*/
/*+
   Stores a node in the btree.  The node is stored in the btree to the
   right of node whose key value is greater than the key value of the node
   and to the left of a node whose key value is less than or equal to the
   key value of the node.  The function 'f' is passed pointers to two keys
   (ie f(key1,key2)), and if key1 is greater than key2 then f should
   return and int of 1.  If key1 is less than key2, then 'f' should return
   and int of -1, and finally if key1 and key2 are equal then 'f' should
   return an int of 0.
-*/
/***************************************************************************/
/* FCN */ bt_storenode(node, f)

nodetype       *node;              /*  (I) pointer to node to store  */
int            (*f)();             /*  (I) pointer to compare function  */
{

nodetype       *left  = NULL;      /*  leftmost node pointer  */
nodetype       *right = NULL;      /*  rightmost node pointer  */


/*  CHECK THE HEAD OF THE TREE FIRST  */

   if (BT_HEAD == NULL) {
       BT_HEAD = node;
       node->l = node->r = node->p = NULL;
       node->flags.ltag = node->flags.rtag = 1;
   }


/*  HEADER IS ALREADY ACTIVE NOW INSERT IN THE BTREE  */

   else {
       for ( bt_cnode=BT_HEAD; bt_cnode!=NULL; ) {
           if ((*f)(bt_cnode->key,node->key) == 1) {
               left = bt_cnode;
               if (bt_cnode->flags.rtag) {
                   node->r             = bt_cnode->r;
                   node->l             = left;
                   node->p             = bt_cnode;
                   node->flags.ltag = node->flags.rtag = 1;
                   bt_cnode->r         = node;
                   bt_cnode->flags.rtag= 0;
                   bt_cnode            = NULL;
               }
               else bt_cnode = bt_cnode->r;
           }
           else {
               right = bt_cnode;
               if (bt_cnode->flags.ltag) {
                   node->l             = bt_cnode->l;
                   node->r             = right;
                   node->p             = bt_cnode;
                   node->flags.ltag = node->flags.rtag = 1;
                   bt_cnode->l         = node;
                   bt_cnode->flags.ltag= 0;
                   bt_cnode            = NULL;
               }
               else bt_cnode = bt_cnode->l;
           }
       }
   }

   return GOOD;
      
/*  END BT_STORENODE FUNCTION  */
}

int bt_true()
{
    return TRUE;
}
