/***************************************************************************
   Module Name:  dxfcodes    sdk_src_general_dxfcodes.h

   Description:  dxf terms include file

   Author:  Tools and Technologies Group
            Autodesk, Inc.
            Software Developers' Kit
   
   (c) Copyright 1989, 1990 by Autodesk Inc.
   ***************************************************************************
      Permission to use, copy, modify, and distribute this software and its
      documentation for the purpose of creating applications for AutoCAD, is
      hereby granted in accordance with the terms of the License Agreement
      accompanying this product.
   ***************************************************************************

      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. AUTODESK 
      SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY OR
      FITNESS FOR A PARTICULAR USE. AUTODESK, INC. DOES NOT WARRANT THAT
      THE OPERATION OF THE PROGRAM WILL BE UNINTERRUPTED OR ERROR FREE.

   ***************************************************************************
                           RESTRICTED RIGHTS LEGEND 

      Use, duplication, or disclosure by the U.S. Government is subject 
      to restrictions set forth in FAR 52.227-19 (Commerical Computer
      Software - Restricted Rights) and DFAR 252.227-7013 (c) (1) (ii)
      (Rights in Technical Data and Computer Software), as applicable.
   ***************************************************************************

   Modification History:

   Notes and restrictions on use:
       This file is designed to be used with all AutoCAD versions up to
       and including Release 10.

***************************************************************************/
/****************************************************************************/
/*  DEFINES  */
/****************************************************************************/

#ifndef DXFCODES
#define DXFCODES    1


/* SELECTION SET FILTERS */

#define DXF_X_FILTER            0   /* X-filter */
#define DXF_LAST_ENTITY         1   /* selecting last entity */
#define DXF_PASSING_PT          2   /* passing point */
#define DXF_WINDOW              3   /* window */
#define DXF_CROSSING            4   /* crossing */

/* ENTITY TYPES CODES FOR ADS */

#define ET_NORM                 1   /* normal entity */
#define ET_SSET                 2   /* selection set */
#define ET_TBL                  3   /* table */
#define ET_VPORT                3   /* viewport table */
#define ET_LTYPE                4   /* line style table */
#define ET_LAYER                5   /* layer table */
#define ET_STYLE                6   /* shape table */
#define ET_VIEW                 7   /* view table */
#define ET_UCS                  8   /* UCS table */
#define ET_BLOCK                9   /* block table */


/* HEADER TYPES CODES */

#define DXF_ACADVER         0       /* Autocad database version number */
#define DXF_ANGBASE         1       /* angle 0 direction */
#define DXF_ANGDIR          2       /* 1=cwise angles, 0=countercwise */
#define DXF_ATTDIA          3       /* attrib entry dialogs,1=on,0=off */
#define DXF_ATTMODE         4       /* attrib vis;0=none,1=normal,2=off */
#define DXF_ATTREQ          5       /* attrib prmpt->INSERT; 1=on,0=off */
#define DXF_AUNITS          6       /* UNITS for for angles */
#define DXF_AUPREC          7       /* UNITS precision for angles */
#define DXF_AXISMODE        8       /* axis on if non-zero */
#define DXF_AXISUNITXY      9       /* axis X/Y tick spacing */
#define DXF_BLIPMODE       10       /* blip mode on if nonzero */
#define DXF_CECOLOR        11       /* ent clr #,0=BYBLOCK,256=BYLAYER */
#define DXF_CELTYPE        12       /* ent ltype name/BYBLOCK/BYLAYER */
#define DXF_CHAMFERA       13       /* first chamfer distance */
#define DXF_CHAMFERB       14       /* second chamfer distance */
#define DXF_CLAYER         15       /* current layer name */
#define DXF_COORDS         16       /* 0=stat crd disp,1=cont upd,2="d<a" fmt */
#define DXF_DIMALT         17       /* alt unit dimen performed if !0 */
#define DXF_DIMALTRD       18       /* alt unit decimal places */
#define DXF_DIMALTF        19       /* alt unit scale factor */
#define DXF_DIMAPOST       20       /* alt dimensioning suffix */
#define DXF_DIMASO         21       /* 1=create assoc dim, 0=drw indiv ent */
#define DXF_DIMASZ         22       /* dimensioning arrow size */
#define DXF_DIMBLK0        23       /* arrow block name */
#define DXF_DIMBLK1        24       /* first arrow block name */
#define DXF_DIMBLK2        25       /* second arrow block name */
#define DXF_DIMCEN         26       /* size of center mark/lines */
#define DXF_DIMDLE         27       /* dimension line extension */
#define DXF_DIMDLI         28       /* dimension line increment */
#define DXF_DIMEXE         29       /* extension line extension */
#define DXF_DIMEXO         30       /* extension line offset */
#define DXF_DIMLFAC        31       /* linear measurements scale factor */
#define DXF_DIMLIN         32       /* dimension lim generated if !0 */
#define DXF_DIMPOST        33       /* general dimensioning suffix */
#define DXF_DIMRND         34       /* round value for dim distances */
#define DXF_DIMSAH         35  /*  use separate arrow blocks if !0           */
#define DXF_DIMSCALE       36  /*  overall dimen scale factor                */
#define DXF_DIMSE1         37  /*  1st extension line                        */
#define DXF_DIMSE2         38  /*  2nd extension line                        */
#define DXF_DIMSHO         39  /*  1=recomp dim while drag, 0=drag org image */
#define DXF_DIMSOXD        40  /*  suppress out-ext dim lines if !0          */
#define DXF_DIMTAD         41  /*  text above dimension line if !0           */
#define DXF_DIMTIH         42  /*  text inside horizontal if !0              */
#define DXF_DIMTIX         43  /*  force text inside exts if !0              */
#define DXF_DIMTM          44  /*  minus tolerance                           */
#define DXF_DIMTOF         45  /*  if text out ext, force line bet ext if !0 */
#define DXF_DIMTOH         46  /*  text outside horix if !0                  */
#define DXF_DIMTOL         47  /*  dimension tolerances gen if !0            */
#define DXF_DIMTP          48  /*  plus tolerances                           */
#define DXF_DIMTSZ         49  /*  dimension tick size,0=no ticks            */
#define DXF_DIMTVP         50  /*  text vertical position                    */
#define DXF_DIMTXT         51  /*  dimension text height                     */
#define DXF_DIMZIN         52  /*  0 suppression for "ft&in" dimens          */
#define DXF_DRAGMODE       53  /*  0=auto,1=on,2=auto                        */
#define DXF_ELEVATE        54  /*  current elevation set by ELEV cmd         */
#define DXF_EXTMAX         55  /*  XY drawing extents upper rt crn           */
#define DXF_EXTMIN         56  /*  XY drawing extents lower lt crn           */
#define DXF_FILLETRAD      57  /*  fillet radius                             */
#define DXF_FILLMODE       58  /*  fill mode on if nonzero                   */
#define DXF_FLATLAND       59  /*  force comp with older vers if nonzero     */
#define DXF_HANDLING       60  /*  handles enabled if nonzero                */
#define DXF_HANDSEED       61  /*  next available handle                     */
#define DXF_INSBASE        62  /*  insertion base                            */
#define DXF_LIMCHECK       63  /*  nonzeo if limits checking is on           */
#define DXF_LIMMAX         64  /*  xy drawing limits upper rt crn            */
#define DXF_LIMMIN         65  /*  xy drawing limits lower lt crn            */
#define DXF_LTSCALE        66  /*  global linetype scale                     */
#define DXF_LUNITS         67  /*  UNITS format for coord & dist             */
#define DXF_LUPREC         68  /*  UNITS precsion for coord & dist           */
#define DXF_MENU           69  /*  name of menu file                         */
#define DXF_MIRRTEXT       70  /*  MIRROR text if !0                         */
#define DXF_ORTHOMODE      71  /*  ORTHO mode on if !0                       */
#define DXF_OSMODE         72  /*  running object snap modes                 */
#define DXF_PDMODE         73  /*  point display mode                        */
#define DXF_PDSIZE         74  /*  point display size                        */
#define DXF_PLINEWID       75  /*  default polyline width                    */
#define DXF_QTEXTMODE      76  /*  quick textmode on if nonzero              */
#define DXF_REGENMODE      77  /*  REGENAUTO mode on if nonzero              */
#define DXF_SKETCHINC      78  /*  sketch record increment                   */
#define DXF_SKPOLY         79  /*  0=sketch lines, 1= sketch plines          */
#define DXF_SPLFRAME       80  /*  spline control polygon disp 1=on 0=off     */
#define DXF_SPLINESEGS     81  /*  # line seg per spline patch               */
#define DXF_SPLINETYPE     82  /*  spline curve type fpr "PEDITSPL"          */
#define DXF_SURFTAB1       83  /*  # mesh tabulations in 1st dir             */
#define DXF_SURFTAB2       84  /*  # mesh tabulations in 2nd dir             */
#define DXF_SURFTYPE       85  /*  surface type for PEDIT SMOOTH             */
#define DXF_SURFU          86  /*  surface density in M direction            */
#define DXF_SURFV          87  /*  surface density in N direction            */
#define DXF_TDCREATE       88  /*  data/time for drawing creation            */
#define DXF_TDINDWG        89  /*  cumulative editing time                   */
#define DXF_TDUPDATE       90  /*  date/time of last drawing update          */
#define DXF_TDUSRTIMER     91  /*  user elapsed time                         */
#define DXF_TEXTSIZE       92  /*  default text height                       */
#define DXF_TEXTSTYLE      93  /*  current text style name                   */
#define DXF_THICK          94  /*  current thickness set by ELEV             */
#define DXF_TRACEWID       95  /*  default trace width                       */
#define DXF_UCSNAME        96  /*  name of current UCS                       */
#define DXF_UCSORG         97  /*  origin of current UCS                     */
#define DXF_UCSXDIR        98  /*  direction of current UCX x-axis           */
#define DXF_UCSYDIR        99  /*  direction of current UCX z-axis           */
#define DXF_USERI1        100  /*  joker user integer variable               */
#define DXF_USERI2        101  /*  joker user integer variable               */
#define DXF_USERI3        102  /*  joker user integer variable               */
#define DXF_USERI4        103  /*  joker user integer variable               */
#define DXF_USERI5        104  /*  joker user integer variable               */
#define DXF_USERR1        105  /*  joker user real variable                  */
#define DXF_USERR2        106  /*  joker user real variable                  */
#define DXF_USERR3        107  /*  joker user real variable                  */
#define DXF_USERR4        108  /*  joker user real variable                  */
#define DXF_USERR5        109  /*  joker user real variable                  */
#define DXF_USRTIMER      110  /*  0 = timer off 1 = timer on                */
#define DXF_WORLDVIEW     111  /*  1=set UCS; 0 = do not change              */
#define DXF_FASTZOOM      112  /*  fast zoom enabled if nonzero              */
#define DXF_GRIDMODE      113  /*  grid mode on if nonzero                   */
#define DXF_GRIDUNIT      114  /*  grid x spacing                            */
#define DXF_SNAPANG       115  /*  snap grid rotation angle                  */
#define DXF_SNAPBASE      116  /*  snap grid base point                      */
#define DXF_SNAPISOPAIR   117  /*  isometric plane;0=lt,1=tp,2=rt            */
#define DXF_SNAPMODE      118  /*  snap mode on if nonzero                   */
#define DXF_SNAPSTYLE     119  /*  snap style;0=std; 1=isometric             */
#define DXF_SNAPUNIT      120  /*  snap grid x spacing                       */
#define DXF_VIEWCTR       121  /*  xy center of current view                 */
#define DXF_VIEWDIR       122  /*  viewing direction                         */ 
#define DXF_VIEWSIZE      123  /*  height of view                            */

/*  GENERAL TYPE CODES  */

#define DXF_ITEM_START          0       /*  start of item -  text follows  */
#define DXF_ITEM_END            0       /*  end of item  */


/*  GENERAL TABLE TYPE CODES  */

#define DXF_TBL_NAME            2       /*  table name  */
#define DXF_TBL_ENTRYCOUNT     70       /*  maximum number of entries  */
#define DXF_TBL_GROUPFLAG      70       /*  table group flag  */


/*  LINE TYPE TABLE CODE DEFINITIONS  */

#define DXF_LT_DESCRIPTION      3       /*  descriptive text  */
#define DXF_LT_LENGTH          40       /*  total pattern length  */
#define DXF_LT_DASHLENGTH      49       /*  length of dashes  */
#define DXF_LT_ALIGNMENT       72       /*  alignment code  */
#define DXF_LT_DASHCOUNT       73       /*  number of dashes  */


/*  LAYER TABLE CODE DEFINITIONS  */

#define DXF_LYR_LINETYPE        6       /*  linetype name  */
#define DXF_LYR_COLOR          62       /*  colour  */


/*  TEXT STYLE TABLE CODE DEFINITIONS  */

#define DXF_STY_FONT            3       /*  font name  */
#define DXF_STY_BIGFONT         4       /*  big font name  */
#define DXF_STY_HEIGHT         40       /*  fixed height  */
#define DXF_STY_WIDTH          41       /*  width factor  */
#define DXF_STY_LASTHEIGHT     42       /*  last height used  */
#define DXF_STY_OBLIQUEANGLE   50       /*  obliquing angle  */
#define DXF_STY_GENERATION     71       /*  text generation flag  */


/*  VIEW TABLE CODE DEFINITIONS  */

#define DXF_VW_CENTER_X        10       /*  center X coordinate  */
#define DXF_VW_CENTER_Y        20       /*  center Y coordinate  */
#define DXF_VW_TARGETDIR_X     11       /*  target X direction  */
#define DXF_VW_TARGETDIR_Y     21       /*  target Y direction  */
#define DXF_VW_TARGETDIR_Z     31       /*  target Z direction  */
#define DXF_VW_TARGETPT_X      12       /*  target X coordinate  */
#define DXF_VW_TARGETPT_Y      22       /*  target Y coordinate  */
#define DXF_VW_TARGETPT_Z      32       /*  target Z coordinate  */
#define DXF_VW_HEIGHT          40       /*  height  */
#define DXF_VW_WIDTH           41       /*  width  */
#define DXF_VW_LENSLENGTH      42       /*  lens length  */
#define DXF_VW_FRONTCLIP       43       /*  front clip offset  */
#define DXF_VW_BACKCLIP        44       /*  back clip offset  */
#define DXF_VW_TWISTANGLE      50       /*  twist angle  */
#define DXF_VW_VIEWMODE        71       /*  viewmode variable  */


/*  UCS TABLE CODE DEFINITIONS  */

#define DXF_UCS_ORIGIN_X       10       /*  origin X coordinate  */
#define DXF_UCS_XAXIS_X        11       /*  x axis X direction  */
#define DXF_UCS_YAXIS_X        12       /*  y axis X direction  */
#define DXF_UCS_ORIGIN_Y       20       /*  origin Y coordinate  */
#define DXF_UCS_XAXIS_Y        21       /*  x axis Y direction  */
#define DXF_UCS_YAXIS_Y        22       /*  y axis Y direction  */
#define DXF_UCS_ORIGIN_Z       30       /*  origin Z coordinate  */
#define DXF_UCS_XAXIS_Z        31       /*  x axis Z direction  */
#define DXF_UCS_YAXIS_Z        32       /*  y axis Z direction  */


/*  VIEWPORT TABLE CODE DEFINITIONS  */

#define DXF_VPRT_LL_X           10       /*  lower left X  */
#define DXF_VPRT_LL_Y           20       /*  lower left Y  */
#define DXF_VPRT_UR_X           11       /*  upper right X  */
#define DXF_VPRT_UR_Y           21       /*  upper right Y  */
#define DXF_VPRT_CNTR_X         12       /*  center X coordinate  */
#define DXF_VPRT_CNTR_Y         22       /*  center Y coordinate  */
#define DXF_VPRT_SNAPBASE_X     13       /*  snap base X point  */
#define DXF_VPRT_SNAPBASE_Y     23       /*  snap base Y point  */
#define DXF_VPRT_SNAPSPACE_X    14       /*  snap X spacing  */
#define DXF_VPRT_SNAPSPACE_Y    24       /*  snap Y spacing  */
#define DXF_VPRT_GRIDSPACE_X    15       /*  grid X spacing  */
#define DXF_VPRT_GRIDSPACE_Y    25       /*  grid Y spacing  */
#define DXF_VPRT_TARGETDR_X     16       /*  target X direction  */
#define DXF_VPRT_TARGETDR_Y     26       /*  target Y direction  */
#define DXF_VPRT_TARGETDR_Z     36       /*  target Z direction  */
#define DXF_VPRT_TARGETPT_X     17       /*  target X coordinate  */
#define DXF_VPRT_TARGETPT_Y     27       /*  target Y coordinate  */
#define DXF_VPRT_TARGETPT_Z     37       /*  target Z coordinate  */
#define DXF_VPRT_HEIGHT         40       /*  height  */
#define DXF_VPRT_ASPECT         41       /*  aspect ratio  */
#define DXF_VPRT_LENSLENGTH     42       /*  lens length  */
#define DXF_VPRT_FRONTOFFSET    43       /*  front clipping offset  */
#define DXF_VPRT_BACKOFFSET     44       /*  back clipping offset  */
#define DXF_VPRT_SNAPROTATION   50       /*  snap rotation angle  */
#define DXF_VPRT_TWIST          51       /*  twist angle  */
#define DXF_VPRT_VIEWMODE       71       /*  viewmode status  */
#define DXF_VPRT_CIRZOOM        72       /*  circle zoom percent  */
#define DXF_VPRT_FASTZOOM       73       /*  fast zoom setting  */
#define DXF_VPRT_UCSICON        74       /*  UCSICON setting  */
#define DXF_VPRT_SNAPMODE       75       /*  snap mode (on/off)  */
#define DXF_VPRT_GRIDMODE       76       /*  grid mode (on/off)  */
#define DXF_VPRT_SNAPSTYLE      77       /*  snap style  */
#define DXF_VPRT_SNAPISOPAIR    78       /*  snap isopair  */


/*  GENERAL ENTITY CODE DEFINITIONS  */

#define DXF_ENTITY_NAME        -1       /*  entity name  */
#define DXF_ENTITY_TYPE         0       /*  entity type  */
#define DXF_ENTITY_HANDLE       5       /*  entity handle  */
#define DXF_LINETYPE            6       /*  linetype name  */
#define DXF_LAYER               8       /*  layer name  */
#define DXF_X(x)           (10+x)       /*  x coordinate ranges 10-19  */
#define DXF_Y(x)           (20+x)       /*  y coordinate ranges 20-29  */
#define DXF_Z(x)           (30+x)       /*  z coordinate ranges 30-39  */
#define DXF_ELEVATION          38       /*  elevation  */
#define DXF_THICKNESS          39       /*  thickness  */
#define DXF_COLOR              62       /*  colour  */
#define DXF_TCODE              76       /*  t_code  */
#define DXF_ATFLAG             78       /*  attribute flag  */
#define DXF_ECS_X             210       /*  ECS X component  */
#define DXF_ECS_Y             220       /*  ECS Y component  */
#define DXF_ECS_Z             230       /*  ECS Z component  */


/*  LINE/3DLINE CODE DEFINITIONS  */

#define DXF_LIN_X_START        10       /*  start X coordinate  */
#define DXF_LIN_Y_START        20       /*  start Y coordinate  */
#define DXF_LIN_Z_START        30       /*  start Z coordinate  */
#define DXF_LIN_X_END          11       /*  end X coordinate  */
#define DXF_LIN_Y_END          21       /*  end Y coordinate  */
#define DXF_LIN_Z_END          31       /*  end Z coordinate  */


/*  POINT CODE DEFINITIONS  */

#define DXF_PT_X               10       /*  X coordinate of point  */
#define DXF_PT_Y               20       /*  Y coordinate of point  */
#define DXF_PT_Z               30       /*  Z coordinate of point  */
#define DXF_PT_XAXIS           50       /*  angle of X axis in UCS  */


/*  CIRCLE CODE DEFINITIONS  */

#define DXF_CIR_CENTER_X       10       /*  center X coordinate  */
#define DXF_CIR_CENTER_Y       20       /*  center Y coordinate  */
#define DXF_CIR_CENTER_Z       30       /*  center Z coordinate  */
#define DXF_CIR_RADIUS         40       /*  radius  */


/*  ARC CODE DEFINITIONS  */

#define DXF_ARC_CENTER_X       10       /*  center X coordinate  */
#define DXF_ARC_CENTER_Y       20       /*  center Y coordinate  */
#define DXF_ARC_CENTER_Z       30       /*  center Z coordinate  */
#define DXF_ARC_RADIUS         40       /*  radius  */
#define DXF_ARC_STARTANGLE     50       /*  start angle  */
#define DXF_ARC_ENDANGLE       51       /*  end angle  */


/*  TRACE CODE DEFINITIONS  */

#define DXF_TRC_X1             10       /*  first X coordinate  */
#define DXF_TRC_Y1             20       /*  first Y coordinate  */
#define DXF_TRC_Z1             30       /*  first Z coordinate  */
#define DXF_TRC_X2             11       /*  second X coordinate  */
#define DXF_TRC_Y2             21       /*  second Y coordinate  */
#define DXF_TRC_Z2             31       /*  second Z coordinate  */
#define DXF_TRC_X3             12       /*  third X coordinate  */
#define DXF_TRC_Y3             22       /*  third Y coordinate  */
#define DXF_TRC_Z3             32       /*  third Z coordinate  */
#define DXF_TRC_X4             13       /*  fourth X coordinate  */
#define DXF_TRC_Y4             23       /*  fourth Y coordinate  */
#define DXF_TRC_Z4             33       /*  fourth Z coordinate  */


/*  SOLID CODE DEFINITIONS  */

#define DXF_SLD_X1             10       /*  first X coordinate  */
#define DXF_SLD_Y1             20       /*  first Y coordinate  */
#define DXF_SLD_Z1             30       /*  first Z coordinate  */
#define DXF_SLD_X2             11       /*  second X coordinate  */
#define DXF_SLD_Y2             21       /*  second Y coordinate  */
#define DXF_SLD_Z2             31       /*  second Z coordinate  */
#define DXF_SLD_X3             12       /*  third X coordinate  */
#define DXF_SLD_Y3             22       /*  third Y coordinate  */
#define DXF_SLD_Z3             32       /*  third Z coordinate  */
#define DXF_SLD_X4             13       /*  fourth X coordinate  */
#define DXF_SLD_Y4             23       /*  fourth Y coordinate  */
#define DXF_SLD_Z4             33       /*  fourth Z coordinate  */


/*  TEXT CODE DEFINITIONS  */

#define DXF_TXT_STRING          1       /*  string of characters  */
#define DXF_TXT_STYLENAME       7       /*  style name  */
#define DXF_TXT_INSERT_X       10       /*  insertion X coordinate  */
#define DXF_TXT_INSERT_Y       20       /*  insertion Y coordinate  */
#define DXF_TXT_INSERT_Z       30       /*  insertion Z coordinate  */
#define DXF_TXT_ALIGN_X        11       /*  alignment X coordinate  */
#define DXF_TXT_ALIGN_Y        21       /*  alignment Y coordinate  */
#define DXF_TXT_ALIGN_Z        31       /*  alignment Z coordinate  */
#define DXF_TXT_HEIGHT         40       /*  height  */
#define DXF_TXT_X_SCALE        41       /*  relative X scale factor  */
#define DXF_TXT_ROTATION       50       /*  rotation angle  */
#define DXF_TXT_OBLIQUE        51       /*  obliquing angle  */
#define DXF_TXT_GENERATION     71       /*  generation flags  */
#define DXF_TXT_JUSTIFICATION  72       /*  justification flag  */


/*  SHAPE CODE DEFINITIONS  */

#define DXF_SHP_NAME            2       /*  name  */
#define DXF_SHP_INSERT_X       10       /*  insertion X coordinate  */
#define DXF_SHP_INSERT_Y       20       /*  insertion Y coordinate  */
#define DXF_SHP_INSERT_Z       30       /*  insertion Z coordinate  */
#define DXF_SHP_SIZE           40       /*  size  */
#define DXF_SHP_X_SCALE        41       /*  relative X scale factor  */
#define DXF_SHP_ROTATION       50       /*  rotation  */
#define DXF_SHP_OBLIQUE        51       /*  obliquing angle  */


/*  BLOCK CODE DEFINITIONS  */

#define DXF_BLK_NAME            2       /*  name  */
#define DXF_BLK_INSERT_X       10       /*  insertion X coordinate  */
#define DXF_BLK_INSERT_Y       20       /*  insertion Y coordinate  */
#define DXF_BLK_INSERT_Z       30       /*  insertion Z coordinate  */
#define DXF_BLK_FLAGS          70       /*  block type flags  */


/*  INSERT CODE DEFINITIONS  */

#define DXF_INS_NAME            2       /*  block name  */
#define DXF_INS_INSERT_X       10       /*  insertion X coordinate  */
#define DXF_INS_INSERT_Y       20       /*  insertion Y coordinate  */  
#define DXF_INS_INSERT_Z       30       /*  insertion Z coordinate  */
#define DXF_INS_X_SCALE        41       /*  X scale factor  */
#define DXF_INS_Y_SCALE        42       /*  Y scale factor  */
#define DXF_INS_Z_SCALE        43       /*  Z scale factor  */
#define DXF_INS_COLUMNSPACE    44       /*  column spacing  */
#define DXF_INS_ROWSPACE       45       /*  row spacing  */
#define DXF_INS_ROTATION       50       /*  rotation  */
#define DXF_INS_ATTRIBUTES     66       /*  attributes follow flag  */
#define DXF_INS_COLUMNCOUNT    70       /*  column count  */
#define DXF_INS_ROWCOUNT       71       /*  row count  */


/*  ATTRIBUTE DEFINITION CODE DEFINITIONS  */

#define DXF_ATD_DEFAULT         1       /*  default value  */
#define DXF_ATD_TAG             2       /*  tag string  */
#define DXF_ATD_PROMPT          3       /*  prompt string  */
#define DXF_ATD_STYLE           7       /*  text style  */
#define DXF_ATD_START_X        10       /*  text start X coordinate  */
#define DXF_ATD_START_Y        20       /*  text start Y coordinate  */
#define DXF_ATD_START_Z        30       /*  text start Z coordinate  */
#define DXF_ATD_ALIGN_X        11       /*  text align X coordinate  */
#define DXF_ATD_ALIGN_Y        21       /*  text align Y coordinate  */
#define DXF_ATD_ALIGN_Z        31       /*  text align Z coordinate  */
#define DXF_ATD_HEIGHT         40       /*  text height  */
#define DXF_ATD_X_SCALE        41       /*  relative X scale factor  */
#define DXF_ATD_ROTATION       50       /*  text rotation  */
#define DXF_ATD_OBLIQUE        51       /*  text obliquing angle  */
#define DXF_ATD_FLAGS          70       /*  attribute flags  */
#define DXF_ATD_GENERATION     71       /*  text generation flags  */
#define DXF_ATD_JUSTIFICATION  72       /*  text justification  */
#define DXF_ATD_FIELDLENGTH    73       /*  field length  */


/*  ATTRIBUTE CODE DEFINITIONS  */

#define DXF_ATT_DEFAULT         1       /*  value  */
#define DXF_ATT_TAG             2       /*  tag string  */
#define DXF_ATT_STYLE           7       /*  text style  */
#define DXF_ATT_START_X        10       /*  text start X coordinate  */
#define DXF_ATT_START_Y        20       /*  text start Y coordinate  */
#define DXF_ATT_START_Z        30       /*  text start Z coordinate  */
#define DXF_ATT_ALIGN_X        11       /*  text align X coordinate  */
#define DXF_ATT_ALIGN_Y        21       /*  text align Y coordinate  */
#define DXF_ATT_ALIGN_Z        31       /*  text align Z coordinate  */
#define DXF_ATT_HEIGHT         40       /*  text height  */
#define DXF_ATT_X_SCALE        41       /*  relative X scale factor  */
#define DXF_ATT_ROTATION       50       /*  text rotation  */
#define DXF_ATT_OBLIQUE        51       /*  text obliquing angle  */
#define DXF_ATT_FLAGS          70       /*  attribute flags  */
#define DXF_ATT_GENERATION     71       /*  text generation flags  */
#define DXF_ATT_JUSTIFICATION  72       /*  text justification  */
#define DXF_ATT_FIELDLENGTH    73       /*  field length  */


/*  POLYGON/POLYLINE CODE DEFINITIONS  */

#define DXF_PLY_START_WIDTH    40       /*  default starting width  */
#define DXF_PLY_END_WIDTH      41       /*  default ending width  */
#define DXF_PLY_VERTICES       66       /*  vertices follow flag  */
#define DXF_PLY_FLAGS          70       /*  polyline flags  */
#define DXF_PLY_M_COUNT        71       /*  polygon mesh M count  */
#define DXF_PLY_N_COUNT        72       /*  polygon mesh N count  */
#define DXF_PLY_M_DENSITY      73       /*  polygon mesh M density  */
#define DXF_PLY_N_DENSITY      74       /*  polygon mesh N density  */
#define DXF_PLY_SMOOTHFLAG     75       /*  smoothed mesh flag  */


/*  VERTEX CODE DEFINITIONS  */

#define DXF_VRT_X              10       /*  X coordinate  */
#define DXF_VRT_Y              20       /*  Y coordinate  */
#define DXF_VRT_Z              30       /*  Z coordinate  */
#define DXF_VRT_START_WIDTH    40       /*  starting width  */
#define DXF_VRT_END_WIDTH      41       /*  ending width  */
#define DXF_VRT_BULGE          42       /*  bulge  */
#define DXF_VRT_TANGENT        50       /*  curve fit tangent direction  */
#define DXF_VRT_FLAGS          70       /*  flags  */


/*  3DFACE CODE DEFINITIONS  */

#define DXF_FAC_X1             10       /*  X coordinate first edge  */
#define DXF_FAC_Y1             20       /*  Y coordinate first edge  */
#define DXF_FAC_Z1             30       /*  Z coordinate first edge  */
#define DXF_FAC_X2             11       /*  X coordinate second edge  */
#define DXF_FAC_Y2             21       /*  Y coordinate second edge  */
#define DXF_FAC_Z2             31       /*  Z coordinate second edge  */
#define DXF_FAC_X3             12       /*  X coordinate third edge  */
#define DXF_FAC_Y3             22       /*  Y coordinate third edge  */
#define DXF_FAC_Z3             32       /*  Z coordinate third edge  */
#define DXF_FAC_X4             13       /*  X coordinate fourth edge  */
#define DXF_FAC_Y4             23       /*  Y coordinate fourth edge  */
#define DXF_FAC_Z4             33       /*  Z coordinate fourth edge  */
#define DXF_FAC_FLAGS          70       /*  flags  */


/*  DIMENSION CODE DEFINITIONS  */

#define DXF_DIM_TEXT            1       /*  dimension text by user  */
#define DXF_DIM_DESCRIPTION     2       /*  anonymous block name  */
#define DXF_DIM_DEFINITION_X   10       /*  x-coordinate of defintion pt*/
#define DXF_DIM_DEFINITION_Y   20       /*  y-coordinate of defintion pt*/
#define DXF_DIM_DEFINITION_Z   30       /*  z-coordinate of defintion pt*/
#define DXF_DIM_MIDDLE_X       11       /*  x-coordinate of middle pt*/
#define DXF_DIM_MIDDLE_Y       21       /*  y-coordinate of middle pt*/
#define DXF_DIM_MIDDLE_Z       31       /*  z-coordinate of middle pt*/
#define DXF_DIM_INSERTION_X    12       /*  x-coordinate of insertion pt*/
#define DXF_DIM_INSERTION_Y    22       /*  y-coordinate of insertion pt*/
#define DXF_DIM_INSERTION_Z    32       /*  z-coordinate of insertion pt*/
#define DXF_DIM_DEF_LIN_ANG_X  13       /*  x-coordinate of lin/ang def pt*/
#define DXF_DIM_DEF_LIN_ANG_Y  23       /*  y-coordinate of lin/ang def pt*/
#define DXF_DIM_DEF_LIN_ANG_Z  33       /*  z-coordinate of lin/ang def pt*/
#define DXF_DIM_DEF_LIN_ANG_X1 14       /*  x-coordinate of insertion pt*/
#define DXF_DIM_DEF_LIN_ANG_Y1 24       /*  y-coordinate of insertion pt*/
#define DXF_DIM_DEF_LIN_ANG_Z1 34       /*  z-coordinate of insertion pt*/
#define DXF_DIM_DEF_DRA_X      15       /*  x-coordinate of def pt for d,r,ang*/
#define DXF_DIM_DEF_DRA_Y      25       /*  y-coordinate of lin/ang def pt*/
#define DXF_DIM_DEF_DRA_Z      35       /*  z-coordinate of lin/ang def pt*/
#define DXF_DIM_DEF_ANG_X      16       /*  x-coordinate of def pt for ang dim*/
#define DXF_DIM_DEF_ANG_Y      26       /*  y-coordinate of def pt for ang dim*/
#define DXF_DIM_DEF_ANG_Z      36       /*  z-coordinate of def pt for ang dim*/
#define DXF_DIM_LEADERLENGTH   40       /*  leader length  */
#define DXF_DIM_ROTATION       50       /*  rotation  */
#define DXF_DIM_HORIZONTAL     51       /*  horizontal rotation  */
#define DXF_DIM_TYPE           70       /*  dimension type  */


/*  ATTRIBUTE GROUP FLAG BIT DEFINITIONS  */

#define DXF_ATD_INVISIBLE    0x01       /*  attribute is invisible  */
#define DXF_ATD_CONSTANT     0x02       /*  attribute is constant  */
#define DXF_ATD_VERIFY       0x04       /*  verification required  */
#define DXF_ATD_PRESET       0x08       /*  preset attribute  */


/*  BLOCK GROUP FLAG BIT DEFINITIONS  */

#define DXF_BLK_ANONYMOUS    0x01       /*  anonymous block  */
#define DXF_BLK_ATTRIBUTES   0x02       /*  block has attributes  */


/*  DIMENSION TYPE DEFINITIONS  */

#define DXF_DIM_ROTATED      0x00       /*  rot, horiz, vert dimension  */
#define DXF_DIM_ALIGNED      0x01       /*  aligned dimension  */
#define DXF_DIM_ANGULAR      0x02       /*  angular dimension  */
#define DXF_DIM_DIAMETER     0x03       /*  diameter dimension  */
#define DXF_DIM_RADIUS       0x04       /*  radius dimension  */
#define DXF_DIM_USERPLACE    0x80       /*  user placement flag  */


/*  3DFACE FLAG BIT DEFINITIONS  */

#define DXF_FAC_EDGE1        0x01       /*  edge 1 invisible  */
#define DXF_FAC_EDGE2        0x02       /*  edge 2 invisible  */
#define DXF_FAC_EDGE3        0x04       /*  edge 3 invisible  */
#define DXF_FAC_EDGE4        0x08       /*  edge 4 invisible  */


/*  POLYLINE FLAG BIT DEFINITIONS  */

#define DXF_PLY_CLOSED       0x01       /*  closed (polygon in M)  */
#define DXF_PLY_CURVEFIT     0x02       /*  curve fit vertices added  */
#define DXF_PLY_SPLINEFIT    0x04       /*  spline fit vertices added  */
#define DXF_PLY_3DLINE       0x08       /*  3D polyline  */
#define DXF_PLY_3DMESH       0x10       /*  3D polygon mesh  */
#define DXF_PLY_CLOSED_N     0x20       /*  closed polygon in N  */


/*  TABLE ENTRY GROUP FLAG BIT DEFINITIONS  (DXF_TBL_GROUPFLAG)  */

#define DXF_TBL_LAYERFROZEN  0x01       /*  layer table frozen bit  */
#define DXF_TBL_TEXTSHAPE    0x01       /*  text table entry is shape  */
#define DXF_TBL_TEXTVERTICAL 0x04       /*  text is vertical  */
#define DXF_TBL_REFERENCED   0x40       /*  table entry referenced bit  */


/*  TEXT/STYLE GENERATION FLAG BIT DEFINITIONS  */

#define DXF_TXT_BACKWARDS    0x02       /*  text is backwards  */
#define DXF_TXT_UPSIDEDOWN   0x04       /*  text is upside down  */


/*  TEXT/STYLE JUSTIFICATION DEFINITIONS  */

#define DXF_TXT_JST_LEFT        0       /*  left justified  */
#define DXF_TXT_JST_CENTER      1       /*  center justified  */
#define DXF_TXT_JST_RIGHT       2       /*  right justified  */
#define DXF_TXT_JST_ALIGN       3       /*  aligned  */
#define DXF_TXT_JST_MIDDLE      4       /*  middle  */
#define DXF_TXT_JST_FIT         5       /*  fit  */


/*  VERTEX FLAG BIT DEFINITIONS  */

#define DXF_VRT_EXTRA        0x01       /*  extra curve fit vertex  */
#define DXF_VRT_CURVEFIT     0x02       /*  curve fit tangent exists  */
#define DXF_VRT_SPLINEVRT    0x08       /*  extra spline fit vertex  */
#define DXF_VRT_SPLINEFRAME  0x10       /*  spline frame control point  */
#define DXF_VRT_3DPOLYLINE   0x20       /*  3D polyline vertex  */
#define DXF_VRT_3DPOLYMESH   0x40       /*  3D polygon mesh vertex  */


/*  VIEWMODE VARIABLE FLAG BIT DEFINITIONS  */

#define DXF_VMD_PERSPECTIVE  0x01       /*  perspective mode  */
#define DXF_VMD_FRONTCLIP    0x02       /*  front clipping active  */
#define DXF_VMD_BACKCLIP     0x04       /*  back clipping active  */
#define DXF_VMD_UCSFOLLOW    0x08       /*  UCS follow mode  */
#define DXF_VMD_FRONTNOTEYE  0x10       /*  front clip not at eye  */



#endif  /*  DXFCODES  */
