/*****************************************************************************
*   Module Name:  edm                      sdk2_src_edm_edm.c                *
*                                                                            *
*   Description:  Entity Data Management Functions Library                   *
*                                                                            *
*   Author:  Tools and Technologies Group                                    *
*            Autodesk, Inc.                                                  *
*            Software Developers' Kit                                        *
*                                                                            *
*   (c) Copyright 1990,1991 by Autodesk Inc.                                 *
******************************************************************************
*   Permission to use, copy, modify, and distribute this software and its    *
*   documentation for the purpose of creating applications for AutoCAD,is    *
*   hereby granted in accordance with the terms of the License Agreement     *
*   accompanying this product.                                               *
******************************************************************************
*                                                                            *
*   AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. AUTODESK     *
*   SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY OR        *
*   FITNESS FOR A PARTICULAR USE. AUTODESK, INC. DOES NOT WARRANT THAT       *
*   THE OPERATION OF THE PROGRAM WILL BE UNINTERRUPTED OR ERROR FREE.        *
*                                                                            *
******************************************************************************
*                           RESTRICTED RIGHTS LEGEND                         *
*                                                                            *
*    Use, duplication, or disclosure by the U.S. Government is subject       *
*    to restrictions set forth in FAR 52.227-19 (Commerical Computer         *
*    Software - Restricted Rights) and DFAR 252.227-7013 (c) (1) (ii)        *
*    (Rights in Technical Data and Computer Software), as applicable.        *
******************************************************************************
*   Modification History:                                                    *
*****************************************************************************/
/****************************************************************************/
/*  INCLUDE FILES  */
/****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "adslib.h"
#include "sdk.h"
#include "dxfcodes.h"


/****************************************************************************/
/*  DEFINES  */
/****************************************************************************/
#define LLL_STORE_END  'E'          /*  lll library end list flag  */
/* Macro to copy a name */
#define copyname(e1, e2)   e1[0] = e2[0]; e1[1] = e2[1]
/* Macro to copy a point */
#define  Cpoint(d, s)   d[X] = s[X];  d[Y] = s[Y];  d[Z] = s[Z]

/****************************************************************************/
/*  TYPEDEFS  */
/****************************************************************************/
/****************************************************************************/
/*  GLOBALS  */
/****************************************************************************/
int AutoLISP = FALSE;               /*  AutoLISP called function flag  */


/****************************************************************************/
/*  FUNCTION DECLARATIONS  */
/****************************************************************************/
extern void *realloc _((void *ptr, size_t size));
extern struct resbuf *lll_alloc_node _((int size));
extern int lll_store_node _((struct resbuf *rb, struct resbuf **rb1,
char type));
int sdk_chkrestype _((struct resbuf *rb,int t));
int sdk_dxf2rbtype _((int dxfcode, int etype));
struct resbuf *sdk_getarg _((int argno, struct resbuf *ab));
int sdk_getargcount _((struct resbuf *argbuf));
int sdk_getenthandle _((ads_enthandle h, struct resbuf *e));
int sdk_getrbangle _((ads_real *a, struct resbuf *rb));
int sdk_getrbcount _((struct resbuf *rb));
int sdk_getrbename _((ads_name *e, struct resbuf *rb));
int sdk_getrblong  _((long *l, struct resbuf *rb));
int sdk_getrborient _((ads_real *o, struct resbuf *rb));
int sdk_getrbpickset _((ads_name *set, struct resbuf *rb));
int sdk_getrb3dpoint _((ads_point pt, struct resbuf *rb));
int sdk_getrbpoint _((ads_point pt, struct resbuf *rb));
int sdk_getrbreal _((ads_real *r, struct resbuf *rb));
int sdk_getrbshort _((int *s, struct resbuf *rb));
int sdk_getrbstrng _((char **s, struct resbuf *rb));
int sdk_isentbuf _((struct resbuf *rb));
int sdk_modrbangle _((struct resbuf *rb, ads_real a));
int sdk_modrbename _((struct resbuf *rb, ads_name ent));
int sdk_modrblong _((struct resbuf *rb, long l));
int sdk_modrborient _((struct resbuf *rb, ads_real o));
int sdk_modrbpickset _((struct resbuf *rb, ads_name set));
int sdk_modrb3dpoint _((struct resbuf *rb, ads_point pt));
int sdk_modrbpoint _((struct resbuf *rb, ads_point pt));
int sdk_modrbreal _((struct resbuf *rb, ads_real r));
int sdk_modrbshort _((struct resbuf *rb, int s));
int sdk_modrbstrng _((struct resbuf *rb, char *s));
struct resbuf *sdk_newrb _((int v));
int sdk_putrbangle _((struct resbuf **head, ads_real a));
int sdk_putrbdxf _((struct resbuf **head, void *v, int type));
int sdk_putrbename _((struct resbuf **head, ads_name e));
int sdk_putrblong _((struct resbuf **head, long l));
int sdk_putrborient _((struct resbuf **head, ads_real o));
int sdk_putrbpickset _((struct resbuf **head, ads_name set));
int sdk_putrb3dpoint _((struct resbuf **head, ads_point p));
int sdk_putrbpoint _((struct resbuf **head, ads_point p));
int sdk_putrbreal _((struct resbuf **head, ads_real r));
int sdk_putrbshort _((struct resbuf **head, int s));
int sdk_putrbstrng _((struct resbuf **head, char *s));
int sdk_putrbval _((struct resbuf **h, union ads_u_val v, int t));
int sdk_relrb _((struct resbuf *rb));

/****************************************************************************/
/*.doc sdk_chkrestype(external)*/
/*+
    This function is used to check the restype of the current item in the
    resbuf passed to it as its first argument.  The second argument, "t", is
    the value to check for.  This type value is the same as is defined in
    the ADS include file (adslib.h) for argument buffers and result buffers,
    and is one of the type values defined in dxfcodes.h for entity buffers.
    The function returns TRUE if the restype matches and FALSE otherwise.
-*/
/****************************************************************************/
int
/* FCN */sdk_chkrestype(rb, t)
struct resbuf *rb;                   /*  (I) resbuf buffer  */
int t;                              /*  (I) restype value  */
{
/*  RETURN THE APPROPRIATE STATUS, BASED ON THE CHECK, TO THE CALLER  */
    return ((rb->restype != t) ? FALSE : TRUE);
}

/****************************************************************************/
/*.doc sdk_dxf2rbtype(external)*/
/*+
    This function converts a DXF code into a `C' type so that the user
    can retrieve the correct value from the resval part of the resbuf
    struct, also, so the user can correctly create an entity buffer.

    The dxfcode argument defines the code to be converted.  The etype
    argument defines the type of entity buffer that provided the code.
    These values are defined in dxfcodes.h as ET_xxxx. The default for
    etype is ET_NORM.
-*/
/****************************************************************************/
int
/* FCN */sdk_dxf2rbtype(dxfcode, etype)
int dxfcode;                        /*  (I) dxf code to be converted  */
int etype;                          /*  (I) entity type  */
{
    int rbtype = RTNONE;            /*  default return state  */
/*  CHECK OUT THE GROUP CODES AND CONVERT  */

    if (dxfcode >= 1000 && dxfcode != 1004 && dxfcode < RTNONE)
          dxfcode %= 1000;          /* XED Codes, hence return normalized */
    if (dxfcode <= 49) {
        if (dxfcode >= 20)          /*  20-49  */
            rbtype = RTREAL;
        else if (dxfcode >= 10) {   /*  10-19  */
            if (etype == ET_VIEW)
               rbtype = RTPOINT;
            else if (etype == ET_VPORT && dxfcode <= 15)
               rbtype = RTPOINT;
            else
               rbtype = RT3DPOINT;
        }
        else if (dxfcode >= 0)
            rbtype = RTSTR;         /*  0-9  */
        else if (dxfcode >= -2)
            rbtype = RTENAME;       /* -2 & -1  */
        else if (dxfcode == -3)
            rbtype = RTSHORT;
    }
    else {
        if (dxfcode <= 59)
           rbtype = RTANG;          /*  50-59  */
        else if (dxfcode <= 79)
           rbtype = RTSHORT;        /*  60-79  */
        else if (dxfcode < 210)     /*  80-209 */
           ;
        else if (dxfcode <= 239)
           rbtype = RT3DPOINT;      /*  210-239  */
        else if (dxfcode == 999)
           rbtype = RTSTR;          /*  999  */
    }
    return rbtype;                  /*  RETURN TYPE TO CALLER  */
}

/****************************************************************************/
/*.doc sdk_getarg(external)*/
/*+
    This function returns a pointer to the specified argument in the
    argument buffer provided.  The first argument in the buffer is 1.
    This function returns NULL upon failure and the result buffer with
    the value upon success.
-*/
/****************************************************************************/
struct resbuf *
/* FCN */sdk_getarg(argno, ab)
int argno;                          /*  (I) number of argument  */
struct resbuf *ab;                  /*  (I) argument buffer  */
{
/*  CHECK THAT THE ARGUMENT COUNT IS IN RANGE  */
    if (sdk_getargcount(ab) < argno)
       return NULL;
    while (--argno)
       ab=ab->rbnext;
    return ab;
}

/****************************************************************************/
/*.doc sdk_getargcount(external)*/
/*+
    This function takes an argument buffer as its only argument and returns
    the number of items in that buffer.  It also sets a variable
    called AutoLISP (which is an int) to the value of TRUE if the argument
    count is not zero, and FALSE otherwise.  This variable can be used by
    the application to determine whether or not the function was called from
    AutoLISP. Often, when a function is called from AutoLISP, it will want
    to return a value to AutoLISP.  This can be detected by the fact that
    arguments were passed to the function (command line execution obviously
    does not allow arguments).  The user can access this variable by declaring
    it as "extern int AutoLISP", in their application.

    If the user is not interested in the setting of this variable, then the
    user can simply call sdk_getrbcount(), which is called by this function.
-*/
/****************************************************************************/
int
/* FCN */sdk_getargcount(argbuf)
struct resbuf *argbuf;              /*  (I) argument buffer  */
{
    int count;                      /*  count of items  */

/*  SET AutoLISP VARIABLE BASED ON RETURN FROM SDK_GETRBCOUNT  */
    AutoLISP = ((count = sdk_getrbcount(argbuf)) == 0) ? FALSE : TRUE;
    return count;
}

/****************************************************************************/
/*.doc sdk_getenthandle(external)*/
/*+
    This function returns the value of the entity handle from the provided
    entity buffer.  If no handle is found, then a BAD value is returned.
    Otherwise, GOOD is returned.
-*/
/****************************************************************************/
int
/* FCN */sdk_getenthandle(h, e)
ads_enthandle h;                    /*  (O) entity handle  */
struct resbuf *e;                   /*  (I) entity buffer  */
{
    for (; e != NULL; e = e->rbnext) {
        if (sdk_chkrestype(e, DXF_ENTITY_HANDLE) == TRUE) {
            strcpy(h,e->resval.rstring);
            return GOOD;
        }
    }
    return BAD;
}
 
/****************************************************************************/
/*.doc sdk_getrbangle(external)*/
/*+
    This function returns the angle stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the angle is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the angle is copied to
    location specified by the "a" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbangle(a, rb)
ads_real *a;                        /*  (O) angle  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTANG) == FALSE)
       return BAD;
    
    *a = rb->resval.rreal;
    
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrbcount(external)*/
/*+
    This function takes a result buffer, entity buffer, or an argument buffer
    as its only argument, and returns the number of items in that buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbcount(rb)
struct resbuf *rb;                  /*  (I) result buffer  */
{
    register int i;                 /*  indexing variable  */

    for (i=0; rb != NULL; rb = rb->rbnext, i++) ;
    return i;
}

/****************************************************************************/
/*.doc sdk_getrbename(external)*/
/*+
    This function returns the entity name stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the entity name is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the entity name is copied to
    location specified by the "e" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbename(e, rb)
ads_name  e;                        /*  (O) entity name  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTENAME) == FALSE)
       return BAD;
    copyname(e, rb->resval.rlname);
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrblong(external)*/
/*+
    This function returns the long int stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the long int is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the long int is copied to
    location specified by the "l" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrblong(l, rb)
long *l;                            /*  (O) long int  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTLONG) == FALSE)
       return BAD;
    *l = rb->resval.rlong;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrborient(external)*/
/*+
    This function returns the orientation stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the orientation is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the orientation is copied to
    location specified by the "o" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrborient(o, rb)
ads_real *o;                        /*  (O) orientation  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTORINT) == FALSE)
       return BAD;
    *o = rb->resval.rreal;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrbpickset(external)*/
/*+
    This function returns the set name stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the set name is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the set name is copied to
    location specified by the "set" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbpickset(set, rb)
ads_name set;                       /*  (O) entity name  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTPICKS) == FALSE)
       return BAD;
    copyname(set, rb->resval.rlname);
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrb3dpoint(external)*/
/*+
    This function returns the 3D point stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the 3D point is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the 3D point is copied to
    location specified by the "pt" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrb3dpoint(pt, rb)
ads_point pt;                       /*  (O) point  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RT3DPOINT) == FALSE)
       return BAD;
    Cpoint(pt, rb->resval.rpoint);
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrbpoint(external)*/
/*+
    This function returns the 2D point stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the 2D point is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the 2D point is copied to
    location specified by the "pt" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbpoint(pt, rb)
ads_point pt;                       /*  (O) point  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTPOINT) == FALSE)
       return BAD;
    copyname(pt, rb->resval.rpoint);
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrbreal(external)*/
/*+
    This function returns the real stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the real is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the real is copied to
    location specified by the "r" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbreal(r, rb)
ads_real *r;                        /*  (O) real value  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTREAL) == FALSE)
       return BAD;
    *r = rb->resval.rreal;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrbshort(external)*/
/*+
    This function returns the short stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the short is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the short is copied to
    location specified by the "s" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbshort(s, rb)
int *s;                             /*  (O) short  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTSHORT) == FALSE)
       return BAD;
    *s = rb->resval.rint;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_getrbstrng(external)*/
/*+
    This function returns the string stored in the current resbuf,
    argbuf, or entity buffer item to the first argument of the function.
    The restype of the item is checked before the string is copied
    and a value of BAD is returned if the restype does not
    match.  If the restype matches, then the string is copied to
    location specified by the "s" argument, and a status of GOOD
    is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_getrbstrng(s, rb)
char **s;                           /*  (O) string  */
struct resbuf *rb;                  /*  (I) resbuf  */
{
    if (sdk_chkrestype(rb,RTSTR) == FALSE)
       return BAD;
    if ((*s = (char *)malloc(strlen(rb->resval.rstring) + 1)) == NULL)
       return BAD;
    strcpy(*s, rb->resval.rstring);
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_isentbuf(external)*/
/*+
    This function returns TRUE if the buffer passed to it is an entity 
    buffer, and FALSE otherwise.
-*/
/****************************************************************************/
int
/* FCN */sdk_isentbuf(rb)
struct resbuf *rb;                  /*  (I) result/entity buffer  */
{
    return ((rb->restype >= RTNONE) ? FALSE : TRUE);
}

/****************************************************************************/
/*.doc sdk_modrbangle(external) */
/*+
    This function modifies the angle stored in the current resbuf, argbuf,
    or entity buffer item to the value in "a". The restype
    of the item is checked before the angle is modified and a value of BAD
    is returned, if the restype does not match. If the restype matches, then
    the angle is modified with the value in the "a" argument, and a status of
    GOOD is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrbangle(rb, a)
struct resbuf *rb;                  /* (I/O) resbuf */
ads_real a;                         /* (I) the real value */
{
    if (sdk_chkrestype(rb, RTANG) == FALSE)
       return BAD;
    rb->resval.rreal = a;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrbename(external) */
/*+
    This function modifies the entity name stored in the current resbuf,
    argbuf, or entity buffer item to the item stored in "e". The restype
    of the item is checked before the entity name is modified and a value
    of BAD is returned if the restype does not match. If the restype
    matches, then the entity name is modified, and a status of GOOD is
    returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrbename(rb, e)
struct resbuf *rb;
ads_name e;
{
   if (sdk_chkrestype(rb, RTENAME) == FALSE)
      return BAD;
   copyname(rb->resval.rlname, e);
   return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrblong(external) */
/*+
    This function modifies the long stored in the current resbuf,
    argbuf, or entity buffer item to the item stored in "l". The restype
    of the item is checked before the long is modified and a value
    of BAD is returned if the restype does not match. If the restype
    matches, then the long is modified, and a status of GOOD is
    returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrblong(rb, l)
struct resbuf *rb;
long l;
{
   if (sdk_chkrestype(rb, RTLONG) == FALSE)
      return BAD;
   rb->resval.rlong = l;
   return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrborient(external) */
/*+
    This function modifies the orientation stored in the current resbuf,
    argbuf, or entity buffer item to the value in "o". The restype
    of the item is checked before the orientation is modified and a value of
    BAD is returned, if the restype does not match. If the restype matches,
    then the orientation is modified with the value in the "o" argument, and a
    status of GOOD is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrborient(rb, o)
struct resbuf *rb;                  /* (I/O) resbuf */
ads_real o;                         /* (I) the real value */
{
    if (sdk_chkrestype(rb, RTORINT) == FALSE)
       return BAD;
    rb->resval.rreal = o;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrbpickset(external) */
/*+
    This function modifies the set name stored in the current resbuf,
    argbuf, or entity buffer item to the item stored in "set". The restype
    of the item is checked before the set name is modified and a value
    of BAD is returned if the restype does not match. If the restype
    matches, then the set name is modified, and a status of GOOD is
    returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrbpickset(rb, set)
struct resbuf *rb;
ads_name set;
{
   if (sdk_chkrestype(rb, RTPICKS) == FALSE)
      return BAD;
   copyname(rb->resval.rlname, set);
   return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrb3dpoint(external) */
/*+
    This function modifies the point stored in the current resbuf, argbuf,
    or entity buffer item to the value in "pt". The restype
    of the item is checked before the point is modified and a value of BAD
    is returned, if the restype does not match. If the restype matches, then
    the point is modified with the value in the "pt" argument, and a status of
    GOOD is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrb3dpoint(rb, pt)
struct resbuf *rb;                  /* (I/O) resbuf */
ads_point pt;                       /* (I) the point value */
{
    if (sdk_chkrestype(rb, RT3DPOINT) == FALSE)
       return BAD;
    Cpoint(rb->resval.rpoint, pt);
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrbpoint(external) */
/*+
    This function modifies the point stored in the current resbuf, argbuf,
    or entity buffer item to the value in "pt". The restype
    of the item is checked before the point is modified and a value of BAD
    is returned, if the restype does not match. If the restype matches, then
    the point is modified with the value in the "pt" argument, and a status of
    GOOD is returned to the user. Note that pt[Z] should be set to 0.0 before
    calling the function.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrbpoint(rb, pt)
struct resbuf *rb;                  /* (I/O) resbuf */
ads_point pt;                       /* (I) the point value */
{
    if (sdk_chkrestype(rb, RTPOINT) == FALSE)
       return BAD;
    copyname(rb->resval.rpoint, pt);
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrbreal(external) */
/*+
    This function modifies the real stored in the current resbuf, argbuf,
    or entity buffer item to the value in "r". The restype
    of the item is checked before the real is modified and a value of BAD
    is returned, if the restype does not match. If the restype matches, then
    the real is modified with the value in the "r" argument, and a status of
    GOOD is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrbreal(rb, r)
struct resbuf *rb;                  /* (I/O) resbuf */
ads_real r;                         /* (I) the real value */
{
    if (sdk_chkrestype(rb, RTREAL) == FALSE)
       return BAD;
    rb->resval.rreal = r;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrbshort(external) */
/*+
    This function modifies the integer stored in the current resbuf, argbuf,
    or entity buffer item to the value in "i". The restype of the item
    is checked before the integer is modified and a value of BAD is
    returned, if the restype does not match. If the restype matches, then the
    integer is modified with the value in the "i" argument, and a status of
    GOOD is returned to the user.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrbshort(rb, i)
struct resbuf *rb;                  /* (I/O) resbuf */
int i;                              /* (I) the integer value */
{
    if (sdk_chkrestype(rb, RTSHORT) == FALSE)
       return BAD;
    rb->resval.rint = i;
    return GOOD;
}

/****************************************************************************/
/*.doc sdk_modrbstrng(external) */
/*+
    This function modifies the string stored in the current resbuf, argbuf,
    or entity buffer item to the value in "str". The restype of the item is
    checked before the string is modified and a value of BAD is returned
    if the restype does not match. If the restype matches, then the string
    is modified reallocating the memory. The status of GOOD is returned to
    the user once the value is modified.
-*/
/****************************************************************************/
int
/* FCN */sdk_modrbstrng(rb, str)
struct resbuf *rb;                  /* (I/O) resbuf */
char *str;                          /* the string value */
{
   if (sdk_chkrestype(rb, RTSTR) == FALSE)
      return BAD;
   rb->resval.rstring = (char *)realloc(rb->resval.rstring, strlen(str) + 1);
   strcpy(rb->resval.rstring, str);
   return GOOD;
}

/****************************************************************************/
/* .doc sdk_newrb(external) */
/*+
    This function is similar to the ads_newrb function. It takes a value "v"
    and returns a result buffer with rb->restype = v. This functions is
    written so that applications can use this library outside of ADS.
-*/
/****************************************************************************/
struct resbuf *
/* FCN */sdk_newrb(v)
int v;
{
    struct resbuf *rb;

    if ((rb = (struct resbuf *)malloc(sizeof (struct resbuf))) == NULL)
	return RETURN_ABNORMAL;
    else {                          /* Initialize the buffer fields */
	rb->restype = v;
	rb->rbnext = NULL;
    }
    return rb;
}

/****************************************************************************/
/*.doc sdk_putrbangle(external) */
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "angle".  The user must provide a pointer to
    a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbangle(head, a)
struct resbuf **head;          /*  (O) pointer to head of buffer  */
ads_real a;                    /*  (I) angle to store in buffer  */
{
    union ads_u_val v;         /*  union of datatypes  */
    v.rreal = a;
    return sdk_putrbval(head,v,RTANG);
}

/****************************************************************************/
/*.doc sdk_putrbdxf(external)*/
/*+
    This function is used to store a dxfcode along with the value associated
    with it. The function automatically deciphers the type of the data if the
    type is a dxfcode. Otherwise, it is that the type information is given.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbdxf(head, v, type)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
void *v;                            /*  (I) value to store in buffer  */
int type;                           /*  (I) type of value to store  */
{
   int code;                        /*  dxf to rb type conversion  */
   struct resbuf *r;                /*  pointer to resbuf  */

   if ((r = (struct resbuf *)lll_alloc_node(sizeof(struct resbuf))) == NULL)
       return RETURN_ABNORMAL;
   code =  (code = sdk_dxf2rbtype(type, ET_NORM)) == RTNONE ? type : code;

   switch (code) {
   case RTANG:
       r->resval.rreal = *(ads_real *) v;
       break;

   case RTORINT:
       r->resval.rreal = *(ads_real *) v;
       break;

   case RTREAL:
       r->resval.rreal = *(ads_real *) v;
       break;

   case RTENAME:
       r->resval.rlname[0] = (*(ads_name *) v)[0];
       r->resval.rlname[1] = (*(ads_name *) v)[1];
       break;

   case RTLONG:
       r->resval.rlong = *(long *) v;
       break;

   case RTPOINT:
       r->resval.rpoint[0] = ((ads_real *) v)[0];
       r->resval.rpoint[1] = ((ads_real *) v)[1];
       break;

   case RT3DPOINT:
       r->resval.rpoint[0] = ((ads_real *) v)[0];
       r->resval.rpoint[1] = ((ads_real *) v)[1];
       r->resval.rpoint[2] = ((ads_real *) v)[2];
       break;

   case  RTSHORT:
       r->resval.rint = *(short *) v;
       break;

   case  RTSTR:
       if((r->resval.rstring = (char *)malloc(strlen(((char *) v)+1))) == NULL) {
         sdk_relrb(r);
         return RETURN_ABNORMAL;
       }
       strcpy(r->resval.rstring, (char *) v);
       break;

   case RTPICKS:
       r->resval.rlname[0] = (*(ads_name *) v)[0];
       r->resval.rlname[1] = (*(ads_name *) v)[1];
       break;

   default:
       break;
   }
   r->restype = type;

/*  STORE THE NODE IN THE LINKED LIST  */

   if (lll_store_node(r, head, LLL_STORE_END) == RETURN_ABNORMAL) {
       sdk_relrb(r);
       return RETURN_ABNORMAL;
   }
   return RETURN_NORMAL;
}

/****************************************************************************/
/*.doc sdk_putrbename(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "ads_name".  The user must provide a pointer to
    a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbename(head, e)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
ads_name e;                         /*  (I) entity name  */
{
    union ads_u_val v;              /*  value buffer  */

    copyname(v.rlname, e);
    return sdk_putrbval(head,v,RTENAME);
}

/****************************************************************************/
/*.doc sdk_putrblong(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "long".  The user must provide a pointer to
    a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrblong(head, l)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
long l;                             /*  (I) long value  */
{
    union ads_u_val v;              /*  union of values  */

    v.rlong = l;
    return sdk_putrbval(head,v,RTLONG);
}

/****************************************************************************/
/*.doc sdk_putrborient(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "orientation".  The user must provide a pointer 
    to a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrborient(head, o)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
ads_real o;                         /*  (I) orientation value  */
{
    union ads_u_val v;              /*  union of values  */

    v.rreal = o;
    return sdk_putrbval(head,v,RTORINT);
}

/****************************************************************************/
/*.doc sdk_putrbpickset(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "ads_name".  The user must provide a pointer to
    a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbpickset(head, set)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
ads_name set;                       /*  (I) entity name  */
{
    union ads_u_val v;              /*  value buffer  */

    copyname(v.rlname, set);
    return sdk_putrbval(head,v,RTPICKS);
}

/****************************************************************************/
/*.doc sdk_putrb3dpoint(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "3D point".  The user must provide a pointer 
    to a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrb3dpoint(head, p)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
ads_point p;                        /*  (I) point value  */
{

    union ads_u_val v;              /*  union of values  */

    Cpoint(v.rpoint, p);
    return sdk_putrbval(head,v,RT3DPOINT);
}

/****************************************************************************/
/*.doc sdk_putrbpoint(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "2D point".  The user must provide a pointer 
    to a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbpoint(head, p)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
ads_point p;                        /*  (I) point value  */
{

    union ads_u_val v;              /*  union of values  */

    copyname(v.rpoint, p);
    return sdk_putrbval(head,v,RTPOINT);
}

/****************************************************************************/
/*.doc sdk_putrbreal(external) */
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "real".  The user must provide a pointer 
    to a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbreal(head, r)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
ads_real r;                         /*  (I) real value  */
{

    union ads_u_val v;              /*  union of values  */

    v.rreal = r;
    return sdk_putrbval(head,v,RTREAL);
}

/****************************************************************************/
/*.doc sdk_putrbshort(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "short".  The user must provide a pointer 
    to a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbshort(head, s)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
int s;                              /*  (I) short value  */
{
    union ads_u_val v;              /*  union of values  */

    v.rint = s;
    return sdk_putrbval(head,v,RTSHORT);
}

/****************************************************************************/
/*.doc sdk_putrbstrng(external)*/
/*+
    This function adds an item to a result buffer, entity buffer, or linked
    list.  The item is of type "string".  The user must provide a pointer 
    to a linked list (which is originally NULL), and is responsible for the
    integrity of the buffer.  The contents of the argument are not checked
    for the correct type.  If the function fails to allocate memory for the
    node in the buffer, then it returns a status of RETURN_ABNORMAL.
    Also, this status is returned if the node cannot be inserted into the
    linked list.  If the operation is successful, then a status of
    RETURN_NORMAL is returned to the caller.

    The Linked List Library (lll) is used by this function to store and
    manipulate the result buffer.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbstrng(head, s)
struct resbuf **head;               /*  (O) pointer to head of buffer  */
char *s;                            /*  (I) pointer to string  */
{

    union ads_u_val v;              /*  union of values  */

    if ((v.rstring = (char *)malloc(strlen(s)+1)) == NULL)
        return RETURN_ABNORMAL;
    strcpy(v.rstring,s);
    return sdk_putrbval(head,v,RTSTR);
}

/****************************************************************************/
/*.doc sdk_putrbval(external)*/
/*+
    This function is used to store a value in a resbuf.  The item type of the
    value is specified by the type argument.  It may be any of the RT type
    codes specified for ADS or any valid group code for a DXF buffer.  The
    second argument is a structure which is defined in sdk.h and is similar
    to the resval portion of the resbuf (or entbuf) structure.  This function
    is eventually called by the other "sdk_putrbxxx" functions.  It allocates
    a node for the resbuf, and then stores the node in the resbuf using the
    Linked List Library functions (LLL).  The sdk_dxf2rbtype function is used
    to convert dxf group codes to RT type codes so that the correct value can
    be stored.  The head of the link list (head) must be NULL when the list
    is empty, and will be updated by this function as necessary.

    The function fails and returns a RETURN_ABNORMAL state if it cannot
    allocate enough memory to create the node (or any space required to store
    a string in the node).  In ALL cases where string types are involved,
    only enough space to store the string is allocated.
-*/
/****************************************************************************/
int
/* FCN */sdk_putrbval(head, v, type)

struct resbuf **head;               /*  (O) pointer to head of buffer  */
union ads_u_val v;                  /*  (I) value to store in buffer  */
int type;                           /*  (I) type of value to store  */
{

   int code;                        /*  dxf to rb type conversion  */
   struct resbuf *rb;               /*  pointer to resbuf  */

   if ((rb = (struct resbuf *)lll_alloc_node(sizeof(struct resbuf))) == NULL)
       return RETURN_ABNORMAL;
   code =  (code = sdk_dxf2rbtype(type, ET_NORM)) == RTNONE ? type : code;
/*  BUILD UP THE NODE  */

   switch (code) {
   case RTANG:
       rb->resval.rreal = v.rreal;
       break;

   case RTORINT:
       rb->resval.rreal = v.rreal;
       break;

   case RTREAL:
       rb->resval.rreal  = v.rreal;
       break;

   case RTENAME:
       copyname(rb->resval.rlname, v.rlname);
       break;

   case RTLONG:
       rb->resval.rlong = v.rlong;
       break;

   case RTPOINT:
       Cpoint(rb->resval.rpoint, v.rpoint);
       break;

   case RT3DPOINT:
       Cpoint(rb->resval.rpoint, v.rpoint);
       break;

   case RTSHORT:
       rb->resval.rint = v.rint;
       break;

   case RTSTR:
       if((rb->resval.rstring=(char *)malloc(strlen(v.rstring)+1)) == NULL) {
          sdk_relrb(rb);
          return RETURN_ABNORMAL;
       }
       strcpy(rb->resval.rstring,v.rstring);
       break;

   case RTPICKS:
       copyname(rb->resval.rlname, v.rlname);
       break;

   default:
       break;
   }
   rb->restype = type;

/*  STORE NODE IN THE LINKED LIST  */

   if (lll_store_node(rb, head, LLL_STORE_END) == RETURN_ABNORMAL) {
       sdk_relrb(rb);
       return RETURN_ABNORMAL;
   }
   return RETURN_NORMAL;
}

/***************************************************************************/
/* .doc sdk_relrb(external) */
/*+
    This function is similar to the function ads_relrb. This function was
    written so that application can use the function without going through
    ADS.
- */
/**************************************************************************/
int
/* FCN */sdk_relrb(rb)
struct resbuf *rb;
{
    struct resbuf *crb, *this;

    this = rb;

    while (this) {
        crb = this;
	if (crb->restype < 5000) {       /* This is not an ADS RTcode */
	   if (crb->restype == 1004)
	      free(crb->resval.rbinary.buf);
           else {
	     if (crb->restype > 1000)
		 crb->restype -= 1000;
             if (((crb->restype & 0xFF) / 10) == 0 &&
		  (crb->resval.rstring != NULL))
		  free(crb->resval.rstring);
           }
        }
	else {
	   if (crb->restype == RTSTR && crb->resval.rstring != NULL)
	      free(crb->resval.rstring);
        }
	this = this->rbnext;
	free(crb);
    }
    return RETURN_NORMAL;
}

/**************************************************************************/
