/***************************************************************************
   Module Name:  gl.h           sdk_src_gl_gl.h

   Description:  Include header files for Geometry Library 

   Author     :  Tools and Technologies Group
                 Autodesk, Inc.
                 Software Developers' Kit

    (C) Copyright 1990, 1991 by Autodesk, Inc.
***************************************************************************
*  Permission to use, copy, modify, and distribute this software and its  *
*  documentation for the purpose of creating applications for AutoCAD, is *
*  hereby granted in accordance with the terms of the License Agreement   *
*  accompanying this product.                                             *
*                                                                         *
***************************************************************************
*                                                                         *
*  AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. AUTODESK   *
*  SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY OR      *
*  FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. DOES NOT WARRANT THAT    *
*  THE OPERATION OF THE PROGRAM WILL BE UNINTERRUPTED OR ERROR FREE.      *
*                                                                         *
***************************************************************************
*                                                                         *
*                       RESTRICTED RIGHTS LEGEND                          *
*                                                                         *
*  Use, duplication, or disclosure by the U.S. Government is subject      *
*  to restrictions set forth in FAR 52.227-19 (Commercial Computer        *
*  Software - Restricted Rights) and DFAR 252.227-7013 (c) (1) (ii)       *
*  (Rights in Technical Data and Computer Software), as applicable.       *
***************************************************************************

   Modification History:

   Notes and restrictions on use:
       
***************************************************************************/
/**************************************************************************/
/*  GLOBAL DEFINES  */
/**************************************************************************/
#define setpt(a,b)  a[0]=b[0]; a[1]=b[1]; a[2]=b[2]
#define PI       3.1415926535897932
#define ORDER    4

/**************************************************************************/
/*  GLOBAL TYPEDEFS  */
/**************************************************************************/

typedef ads_real   matNxN[ORDER][ORDER];
typedef ads_real   txform[4][4];
typedef struct linfo {                          /* Line information */
                      ads_point pt1,pt2,cos;
                      ads_real  g,h,i;
                     } ltype; 
typedef struct circ  {                          /* Circle Data */
                      ads_point pt;
                      ads_real  radius;
                     } ctype; 
typedef struct conic {                          /* Conic Data */
                      ads_real  a, b, c, 
                                d, e, f;
                     } cotype; 
typedef struct pvtx {
                     struct pvtx    *next;
                     ads_point      vtx;
                     ads_real       bulge;
                    } pvtype; 
typedef struct p2lines {
                        struct p2lines    *next;
                        int               bf;
                        ltype             line;
                        ctype             circle;
                       } pltype; 

/**************************************************************************/
/*  EXTERNAL FUNCTION DECLARATION  */
/**************************************************************************/

extern int       gl_ac2wc     _((txform m, ads_point ndir));
extern ads_real  gl_ang_ptcirc _((ads_point pt, ctype *circle));
extern ads_real  gl_ang_txf2d _((txform m));
extern int       gl_arbaxis   _((ads_point oaxis, ads_point iaxis));
extern int       gl_bul2cir   _((ctype *circle, ads_point p1, ads_point p2,
                                 ads_real bf));
extern int       gl_cir2eli   _((cotype *conic, txform mat, ctype *circle));
extern int       gl_cr_entpt  _((ads_point pt));
extern ads_real  gl_dpl       _((ads_point pt, ltype *line));
extern ads_real  gl_dpp       _((ads_point p1, ads_point p2));
extern int       gl_dist_ptpln _((ads_real *dist, ads_point pt, ads_point pt1,
                                  ads_point pt2, ads_point pt3));
extern int       gl_eang_ptcirc _((ads_real angle[2], ads_point pt,
                                   ctype *circle));
extern int       gl_ec2wc     _((ads_point ap, ads_point bp, ads_point nvec,
                                 int disp));
extern int       gl_ec2uc     _((ads_point ap, ads_point bp, ads_point nvec,
                                 int disp));
extern int       gl_ec2dc     _((ads_point ap, ads_point bp, ads_point nvec,
                                 int disp));
extern ads_real  gl_fixang    _((ads_real angle));
extern int       gl_getline   _((ads_name ename, ltype *line, ads_point nvec));
extern int       gl_getcircle _((ads_name ename, ctype *circle, ads_point nvec));
extern int       gl_getpvtx   _((ads_name ename));
extern int       gl_getrpvtx  _((ads_name ename));
extern int       gl_getpline  _((ads_name ename));
extern int       gl_getrpline _((ads_name ename));
extern int       gl_get_line  _((char *prompt, ads_name ename, ltype *line,
                                 ads_point nvec));
extern int       gl_get_circle _((char *prompt, ctype *circle, ads_point nvec));
extern int       gl_get_pline _((char *prompt));
extern int       gl_set_rpline _((char *prompt));
extern int       gl_intlc_pln _((ads_point pts[2], ltype *line, ctype *circle));
extern int       gl_intlc_spc _((ads_point pts[2], ltype *line, ctype *circle,
                                 ads_point nvl, ads_point nvec));
extern int       gl_intlc_proj _((ads_point pts[2], ltype *line, ctype *circle,
                                  ads_point nvec));
extern int       gl_intcc_pln _((ads_point pts[2], ctype *circ1, ctype *circ2));
extern int       gl_intcc_spc _((ads_point pts[2], ctype *circ1, ctype *circ2,
                                 ads_point nv1, ads_point nv2));
extern int       gl_intll_coef _((ads_point pt, ltype *line1, ltype *line2));
extern int       gl_int_ll    _((ads_point pt, ltype *line1, ltype *line2));
extern int       gl_int_pline _((ltype *line1));
extern int       gl_int_lco   _((ads_point pts[2], ltype *line, cotype *conic));
extern int       gl_line      _((ltype *line, ads_point p1, ads_point p2));
extern int       gl_line_coef _((ltype *line, ads_point p1, ads_point p2));
extern int       gl_ludcmp    _((matNxN m, int indx[ORDER], ads_real *d));
extern void      gl_lubksb    _((matNxN m, int indx[ORDER], ads_real b[ORDER]));
extern void      gl_matXmat   _((txform ma, txform mb, txform mc));
extern ads_real  gl_mat_det   _((matNxN m));
extern int       gl_mat_inv   _((matNxN invm, matNxN m));
extern void      gl_mat_ident _((txform m));
extern void      gl_mat_trans _((txform m, ads_real x, ads_real y,
                                 ads_real z));
extern void      gl_mat_rotx  _((txform m, ads_real theta));
extern void      gl_mat_roty  _((txform m, ads_real theta));
extern void      gl_mat_rotz  _((txform m, ads_real theta));
extern void      gl_mat_multi _((ads_point ap, ads_point bp, txform m,
                                 int disp));
extern void      gl_mat_scal  _((txform ma, ads_real b, txform mc));
extern int       gl_pline_ptarc _((pltype *plnode, ads_point pts[2], int n));
extern int       gl_pln_pts   _((ads_point pln, ads_point p1, ads_point p2,
                                 ads_point p3));
extern int       gl_pln_vec   _((ads_point pln, ads_point v1, ads_point v2));
extern int       gl_ptarc     _((ads_real starta, ads_real enda, ads_point pt,
                                 ctype *circle));
extern int       gl_ptseg     _((ltype *line1, ads_point pt));
extern int       gl_pts_cross _((ads_point nv, ads_point ap, ads_point bp,
                                 ads_point cp));
extern int       gl_pt_pq2l2d _((ads_point pqpt, ads_point pt, ltype *line));
extern int       gl_pt_pq2l3d _((ads_point pqpt, ads_point pt, ltype *line));
extern int       gl_pt_pq2pln _((ads_point pqpt, ads_point pt, ads_point pt1,
                                 ads_point pt2, ads_point pt3));
extern void      gl_pt_oncirc _((ads_point pt, ctype *circle, ads_real angle));
extern int       gl_pt_nstpl  _((ads_point npt, ads_point pt, ltype *line));
extern int       gl_p2lines   _(());
extern int       gl_rpline    _((ads_name ename));
extern int       gl_transform _((txform m, struct resbuf *rbf, struct resbuf *rbt));
extern int       gl_txf_dc2pc _((txform m));
extern int       gl_txf_dc2wc _((txform m));
extern int       gl_txf_ec2wc _((txform m, ads_name ename));
extern int       gl_txf_pc2dc _((txform m));
extern void      gl_txf_rotpt _((txform m, ads_point pt, ads_real theta));
extern int       gl_txf_wc2dc _((txform m));
extern int       gl_txf_wc2ec _((txform m, ads_name ename));
extern int       gl_uc2wc     _((ads_point ap, ads_point bp, int disp));
extern int       gl_uc2ec     _((ads_point ap, ads_point bp, ads_point nvec,
                                 int disp));
extern int       gl_uc2dc     _((ads_point ap, ads_point bp, int disp));
extern void      gl_vec_sub   _((ads_point ap, ads_point bp, ads_point cp));
extern void      gl_vec_scal  _((ads_point ap, ads_real b, ads_point cp));
extern ads_real  gl_vec_abs   _((ads_point ap));
extern ads_real  gl_vec_dot   _((ads_point ap, ads_point bp));
extern int       gl_vec_cross _((ads_point ap, ads_point bp, ads_point cp));
extern void      gl_vec_add   _((ads_point ap, ads_point bp, ads_point cp));
extern ads_real  gl_vec_sqabs _((ads_point ap));
extern int       gl_vec_unit  _((ads_point uv, ads_point ap));
extern int       gl_vec_eqv   _((ads_point v1, ads_point v2));
extern int       gl_vec_ang   _((ads_real *ang, ads_point va, ads_point vb,
                                 ads_point vc));
extern int       gl_wc2uc     _((ads_point ap, ads_point bp, int disp));
extern int       gl_wc2ec     _((ads_point ap, ads_point bp, ads_point nvec,
                                 int disp));
extern int       gl_wc2dc     _((ads_point ap, ads_point bp, int disp));
extern int       gl_wc2ac     _((txform m, ads_point ndir));
