/*\
 *  Autocad ADS library C++ link
 *  $Id: adsapp.hpp%v 1.2 1994/11/13 02:32:39 leon Exp leon $
\*/

#ifndef _ADSAPP_HPP_INCLUDED

#include <iostream.h>
#include <afc/Strings.h>
#include <stl/set.h>
#include <stl/vector.h>
#include <ads/adslib.h>
#include <ads/ads.h>
#include <ads/adsdlg.h>

extern "C" {
   void main(int, char *[]);
}

extern void dispatch(ads_callback_packet *cbpkt);

/**
 ** ADSObject je objekt, ki je osnova za ostale razrede
 **/
class ADSObject
{

public:

    virtual ~ADSObject() {}
    virtual char *nameOf() const = 0;
    virtual int isEqual( const ADSObject & ) const = 0;

    virtual int isSortable() const
        {
        return 0;
        }

    virtual int isAssociation() const
        {
        return 0;
        }

    virtual void printOn( ostream & ) const = 0;

    static ADSObject  *ZERO;

    static ADSObject & ptrToRef( ADSObject *p )
        { return p == 0 ? *ZERO : *p; }
};


inline ostream & operator << ( ostream & out, const ADSObject &obj )
{
    obj.printOn( out );
    return out;
}

/** {secret}
 ** look for: Best C/C++ Tips Ever - tip 130
 **/
class MemberFunction
{
public:
  virtual void Call() =  0;
};

/** {secret} **/
template<class Type> class Callback : public MemberFunction
{
public:

  Callback(Type * pInstance, void (Type::*pFunc)())
    : pInstance(pInstance), pFunc(pFunc){};

  void Call(){ (pInstance->*pFunc)(); }

private:

  Type* pInstance;

  void(Type::*pFunc)();
};


/**
 ** ADSFunction so funcije, ki jih registriramo v ACAD-u
 **/
class ADSFunction 
{
public:
  Strings name;
  short number;
  void (*pFunc)();

  ADSFunction(){}
  ADSFunction(void (*pFunc)(), const char *name, short number);
  void load()
    {
      if ( !ads_defun(name, number))
        cout << "Napaka pri definiranju funkcije" << name << endl;
    }

  void unload()
    {
      ads_undef (name, number);
    }

  void execute()
    {
      (pFunc)();
    }
};

/**
 ** Osnovni razred, ki omogoca registriranje in izvajanje aplikacije
 ** v ACAD kot .exp program.
 **/
class ADSApplication
{
private:
  short total;
  vector<ADSFunction> fArray;

public:
  ADSApplication() {total = 0;}
  ADSApplication(const char *appName) {Register(appName); total = 0;}

protected:
  virtual short load();
  virtual short unload();
  virtual short execute();
  virtual void initialize(){};

public:
  void Register(void (*func)(), const char *funcName);
  void Register(const char *appName);
  void Run(int argc, char *argv[]);
};


/**
 ** Skupini razredov (Button, ListBox, EditBox), ki upravljajo z
 ** Dialog okni v Autocad-u imajo skupni razred Control.
 **/
class Control
{
public:
  CLIENTFUNC callback;
  Strings key;
  int cmdId;
public:
  Control (const char *key,  int cmdId=0, CLIENTFUNC callback = dispatch);
  Control () {};
  void addTile(ads_hdlg hdlg) 
   { ads_action_tile(hdlg, (char *)(const char *)key, callback);}
  virtual ~Control(){}
  virtual void paint(ads_hdlg hdlg){}
  char * getKey(){return (char *)(const char *)key;}
};


    
/**
 ** Omogoca kontrolo nad gumbi v Dialog oknu.
 **/
class Button : public Control
{
public:
  Button(const char *key, int cmdId) : Control(key, cmdId) {};
};


/**
 ** Razred ListBox vsebuje vec stringov (Item), ki so lahko poljubne
 ** dolzine.
 **/
class Item // : public WCSLink 
{
public:
  Strings item;

public:
  Item (const char *item);
  Item(){};
  Item(const Item& i){item = i.item;};
  static void paint(Item i);
};

/** {duh} **/
inline Item::Item (const char *item)
    {
      Item::item = item;
    }


/**
 ** Seznam ListBox je mogoce uporabiti v Dialog oknih kot staticni
 ** izbirni seznam, ali kot padajoci /popup/ izbirni seznam.
 **/
class ListBox : public Control
{
  vector<Item> items;

public:
  ListBox(const char *key, int cmdId) : Control(key, cmdId) {};
  ListBox(const ListBox& listbox);
//  ~ListBox();

  void add(const char *data);
  void newList();
  void paint(ads_hdlg hdlg);
  const char *GetString(int i);

};



/**
 ** EditBox uporabimo za vnos stringov v v Dialog Oknih.
 **/
class EditBox : public Control
{
public:
  Strings text;
  Strings defaultText;

  EditBox(const char *key, int cmdId, const char *defaultText = "");
  EditBox(const EditBox& eb){text=eb.text; defaultText=eb.defaultText;}
  void Set(const char *text);
protected:
  void paint(ads_hdlg hdlg);
};

/**
 ** Dialog okna v AutoCad-u kreiramo s tem razredom, ki je lahko osnova
 ** za ostale (izpeljane) razrede.
 **/
class Dialog
{
private:
  ads_hdlg hdlg;
  Strings file;
  Strings dialogKey;
  Strings helpTopic;
  int dlgId;
  int status;
  static Dialog *current;
  Dialog *previous;

private:
  set<Control *, less <Control *> > controls;

public:
  Dialog(const char *filename, const char *dialogKey, 
         const char *topic = NULL);
  virtual ~Dialog();
  void Register(CLIENTFUNC tileFunc, const char *key);
  virtual void Ok() {};
  virtual void Cancel() {};
  virtual void HandleMsg() {};  // User messages for ads_done_dialog()
  virtual void StartHook() {}; 
  void insert(Control *control);
  virtual void Run();

protected:
   friend void dispatch(ads_callback_packet *cbpkt);
   virtual void HandleMsg(int cmdId, ads_callback_packet *cbpkt);
};


#if 1
/**
 ** Omogoca izbor enega elementa iz seznama ListBox, ki je podan s
 ** tabelo /table/.
 **/
class PickList : protected Dialog
{
private:
  ListBox *lb;
  int selected;

public:
  PickList(const char *filename, const char *dialogKey,
                   const char *table[], int size);
  int Pick();

protected:
  virtual void fillListBox(const char *table[], int size);
  void HandleListBoxMsg(int , ads_callback_packet *cbpkt);
};

/** {secret} **/

#endif


#define _ADSAPP_HPP_INCLUDED
#endif


