/*\
 *     PRIVATE PROFILE STRINGS - DDJ9403 pp 91-92
 *     Vsi profile kot tudi sekcije so "Case sensitive"!
 *     $Id: PROFPORT.C 1.1 1995/01/16 12:13:05 leon Exp $
\*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#else
#include <io.h>
#endif
#include "profport.h"

/* FCN read_line
   Bere eno vrstico iz datoteke fp v buffer bp.
   Vrne TRUE(1) ob uspehu, sicer FALSE(0)
*/
#if 0
static int 
read_line (FILE *fp, char *bp)
{
  int c = '\0';
  int i = 0;
  /* read one line from the source file */
  while ((c = getc (fp)) != '\n')
    {
      if (c == EOF)		/* return on unexpected EOF */
	return (0);
      bp[i++] = (char)c;
    }
  bp[i] = '\0';
  return (1);
}
#endif

/*FCN get_private_profile_int()
  <char *> section - the name of the section to search for
  <char *> entry - the name of the entry to find the value of
  <int> def - the default value in the event of a failed read
  <char *> file_name - the name of the ini file to read from
  Returns: the value located at entry
*/
int 
get_private_profile_int (char *section, char *entry, int def,
			 char *file_name)
{
  FILE *fp;
  char buff[MAX_LINE_LENGTH];
  char *ep;
  char t_section[MAX_LINE_LENGTH];
  char value[17];
  int len;
  int i;

  fp = fopen (file_name, "r");
  if (!fp)
    return (0);
  len = strlen (entry);
  sprintf (t_section, "[%s]", section);	/* Format the section name */

  /* Move through file in file 1 line at a time until
     a section is matched or EOF */
  do
    {
      if (!read_line (fp, buff))
	{
	  fclose (fp);
	  return (def);
	}
  } while (strcmp (buff, t_section));

  /* Now that the section has been found, find the entry.
   * Stop searching upon leaving the section's area */
  do
    {
      if (!read_line (fp, buff) || buff[0] == '\0')
	{
	  fclose (fp);
	  return (def);
	}
  } while (strncmp (buff, entry, len));

  ep = strchr (buff, '=');	/* Parse out the equal sign */
  ep++;
  if (!strlen (ep))		/* No setting */
    return (def);

  /* Copy only numbers fail on characters */
  for (i = 0; isdigit (ep[i]); i++)
    value[i] = ep[i];
  value[i] = '\0';
  fclose (fp);
  return (atoi (value));
}


/*FCN get_private_profile_string()
  <char *> section - the name of the section to search for
  <char *> entry - the name of the entry to find the value of
  <char *> def - default string in the event of a failed read
  <char *> buffer - a pointer to the buffer to copy into
  <int> buffer_len - the max number of characters to copy
  <char *> file_name - the name of the ini file to read from
  Returns: the number of characters copied into the suplied buffer
*/
int 
get_private_profile_string (char *section, char *entry, char *def,
			    char *buffer, int buffer_len,
			    char *file_name)
{
  FILE *fp;
  char buff[MAX_LINE_LENGTH];
  char *ep;
  char t_section[MAX_LINE_LENGTH];
  int len;

  fp = fopen (file_name, "r");
  if (!fp)
    return (0);

  len = strlen (entry);
  sprintf (t_section, "[%s]", section);	/* Format the section name */

  /* Move through file in file 1 line at a time until
     a section is matched or EOF */
  do
    {
      if (!read_line (fp, buff))
	{
	  fclose (fp);
	  strncpy (buffer, def, buffer_len);
	  return (strlen (buffer));
	}
  } while (strcmp (buff, t_section));

  /* Now that the section has been found, find the entry.
   * Stop searching upon leaving the section's area */
  do
    {
      if (!read_line (fp, buff) || buff[0] == '\0')
	{
	  fclose (fp);
	  strncpy (buffer, def, buffer_len);
	  return (strlen (buffer));
	}
  } while (strncmp (buff, entry, len));

  ep = strchr (buff, '=');	/* Parse out the equal sign */
  ep++;

  /* Copy up to buffer_len chars to buffer */
  strncpy (buffer, ep, buffer_len - 1);
  buffer[buffer_len] = '\0';
  fclose (fp);
  return (strlen (buffer));
}



/*FCN write_private_profile_string()
  <char *> section - the name of the section to search for
  <char *> entry - the name of the entry to find the value of
  <char *> buffer - a pointer to the buffer to copy into
  <int> buffer_len - the max number of characters to copy
  <char *> file_name - the name of the ini file to read from
  Returns: TRUE if successful, otherwise FALSE
*/
int
write_private_profile_string (char *section, char *entry,
                              char *buffer,  char *file_name)
{
  FILE *rfp, *wfp;
  char tmp_name[L_tmpnam];
  char buff[MAX_LINE_LENGTH];
  char t_section[MAX_LINE_LENGTH];
  int len;

  len = strlen (entry);
  tmpnam (tmp_name);		/* Get a temporary file name to copy to */
  sprintf (t_section, "[%s]", section);	/* Format the section name */

  if ((rfp = fopen (file_name, "r")) == NULL)
    {				/* If the .ini doesn't exist */
      if ((wfp = fopen (file_name, "w")) == NULL)	/* then make one */
	return (0);
      fprintf (wfp, "%s\n", t_section);
      fprintf (wfp, "%s=%s\n", entry, buffer);
      fclose (wfp);
      return (1);
    }

  if ((wfp = fopen (tmp_name, "w")) == NULL)
    {
      fclose (rfp);
      return (0);
    }

  /* Move through the file one one line at a time until a section is
   * matched or until EOF. Copy to temp file as it is read. */

  do
    {
      if (!read_line (rfp, buff))
	{
	  fprintf (wfp, "\n%s\n", t_section);
	  fprintf (wfp, "%s=%s\n", entry, buffer);
	  /* Clean up and rename */
	  fclose (rfp);
	  fclose (wfp);
	  unlink (file_name);
	  rename (tmp_name, file_name);
	  return (1);
	}
      fprintf (wfp, "%s\n", buff);
  } while (strcmp (buff, t_section));
  /* Now that the section has been found, find the entry.
   * Stop searching upon leaving the section's area.
   * Copy the file as it is read and create entry if one is not found */

  while (1)
    {
      if (!read_line (rfp, buff))
	{
	  /* EOF without an entry so make one */
	  fprintf (wfp, "%s=%s\n", entry, buffer);
	  /* Clean up and rename */
	  fclose (rfp);
	  fclose (wfp);
	  unlink (file_name);
	  rename (tmp_name, file_name);
	  return (1);
	}
      if (!strncmp (buff, entry, len) || buff[0] == '\0')
	break;
      fprintf (wfp, "%s\n", buff);
    }

  if (buff[0] == '\0')
    {
      fprintf (wfp, "%s=%s\n", entry, buffer);
      do
	{
	  fprintf (wfp, "%s\n", buff);
      } while (read_line (rfp, buff));
    }
  else
    {
      fprintf (wfp, "%s=%s\n", entry, buffer);
      while (read_line (rfp, buff))
	{
	  fprintf (wfp, "%s\n", buff);
	}
    }


  /* Clean up and rename */
  fclose (wfp);
  fclose (rfp);
  unlink (file_name);
  rename (tmp_name, file_name);
  return (1);
}
