// Minimal string classes, derived from vector<char>:
//   string: sequence of non-white-space chacters
//   alpha: sequence of aphabetic or dash or apostrophe characters
//   line: sequence of non-newline characters

#ifndef MSTRING_H
#define MSTRING_H
#include <vector.h>
#include <string.h>  

class string : public vector<char> {
public:
        string() : vector<char>() {}
        string(size_t n) : vector<char>(n+1) {}
        string(size_t n, const char& c) : vector<char>(n+1, c) {
                 *(end()-1) = 0;
        }
        string(const char* first, const char* last)
                 : vector<char>(first, last) { insert(end(), (char)0); }
        string(const char* first) 
                 : vector<char>(first, first + strlen(first)+1) {}
        string& operator=(const char* first) {
                 reserve(strlen(first)+1);
                 finish = copy(first, first + strlen(first) + 1, start);
                 return *this;
        }
        const char* chars() const { return start; }
        string& operator=(const string& x) {
             vector<char>::operator=(x);
             return *this;
         }
};

inline void destroy(string* pointer) {
    pointer->string::~string();
}

ostream& operator<<(ostream& o, const string& s) {
        copy(s.begin(), s.end()-1, ostream_iterator<char>(o));
        return o;
}

istream& operator>>(istream& i, string& s) {
        s.erase(s.begin(), s.end());
        while (true) {
                 char c;
                 i.get(c);
                 if (!i.operator void*()) break;
                 if (!(c == ' ' || c == '\n' || c == '\t')) {
                         s.insert(s.end(), c);
                         while (true) {
                                 i.get(c);
                                 if (!i.operator void*()) break;
                       if (!(c == ' ' || c == '\n' || c == '\t')) 
                                         s.insert(s.end(), c);
                                 else 
                                         break;
                         }
                         break;
                 } 
        }
        s.insert(s.end(), (char)0);
        return i;
}

class alpha : public string {
public:
        alpha() : string() {}
        alpha(size_t n) : string(n) {}
        alpha(size_t n, const char& c) : string(n, c) {}
        alpha(const char* first, const char* last) 
                : string(first, last) {}
        alpha(const char* first) : string(first) {}
        alpha& operator=(const char* first) { 
                 reserve(strlen(first)+1);
                 finish = copy(first, first + strlen(first) + 1, start);
                 return *this;
        }
         alpha& operator=(const alpha& x)  {
                 string::operator=(x);
                 return *this;
         } 
};

inline void destroy(alpha* pointer) {
    pointer->alpha::~alpha();
}

ostream& operator<<(ostream& o, const alpha& s) {
        copy(s.begin(), s.end()-1, ostream_iterator<char>(o));
        return o;
}

istream& operator>>(istream& i, alpha& s) {
        s.erase(s.begin(), s.end());
        while (true) {
                 char c;
                 i.get(c);
                 if (!i.operator void*()) break;
                 if (c >= 'A' && c <= 'Z' || c >= 'a' && c <= 'z'
                         || c == '-' || c == '\'') {
                         s.insert(s.end(), c);
                         while (true) {
                                 i.get(c);
                                 if (!i.operator void*()) break;
                                 if (c >= 'A' && c <= 'Z' || 
                                      c >= 'a' && c <= 'z' ||
                                      c == '-' || c == '\'')
                                         s.insert(s.end(), c);
                                 else 
                                         break;
                         }
                         break;
                 } 
        }
        s.insert(s.end(), (char)0);
        return i;
}

class line : public vector<char> {
public:
        line() : vector<char>() {}
        line(size_t n) : vector<char>(n+1) {}
        line(size_t n, const char& c) : vector<char>(n+1, c) {
                 *(end()-1) = 0;
        }
        line(const char* first, const char* last)
                 : vector<char>(first, last) { insert(end(), (char)0); }
        line(const char* first) 
                 : vector<char>(first, first + strlen(first)+1) {}
        line& operator=(const char* first) {
                 reserve(strlen(first)+1);
                 finish = copy(first, first + strlen(first) + 1, start);
                 return *this;
        }
        const char* chars() const { return start; }
        line& operator=(const line& x) {
             vector<char>::operator=(x);
             return *this;
         }
};

inline void destroy(line* pointer) {
    pointer->line::~line();
}

ostream& operator<<(ostream& o, const line& s) {
        copy(s.begin(), s.end()-1, ostream_iterator<char>(o));
        return o;
}

istream& operator>>(istream& i, line& s) {
        s.erase(s.begin(), s.end());
        while (true) {
                 char c;
                 i.get(c);
                 if (!i.operator void*()) break;
                 s.insert(s.end(), c);
                 if (c == '\n') break;
        }
        s.insert(s.end(), (char)0);
        return i;
}

inline void iter_swap(vector<char>* a, vector<char>* b) {
    (*a).swap(*b);
}

inline void iter_swap(string* a, string* b) {
    (*a).swap(*b);
}

inline void iter_swap(alpha* a, alpha* b) {
    (*a).swap(*b);
}

inline void iter_swap(line* a, line* b) {
    (*a).swap(*b);
}

bool operator<(const string& a, const string& b) {
   return strcmp(a.chars(), b.chars()) < 0;
}

inline bool operator<(const alpha& a, const alpha& b) {
   return strcmp(a.chars(), b.chars()) < 0;
}

inline bool operator<(const line& a, const line& b) {
   return strcmp(a.chars(), b.chars()) < 0;
}

#endif
